//---------------------------------------------------------------------
// Copyright(c) 1999 arton
//
#include <vcl.h>
#include <winsock2.h>
#pragma hdrstop

#include "FreePeekStrs.h"
#include "AddressSet.h"
//---------------------------------------------------------------------
#pragma link "IPMaskEdit"
#pragma link "MACCAPV2Lib_OCX"
#pragma resource "*.dfm"
TAddressInputDlg *AddressInputDlg;
//---------------------------------------------------------------------
static const int cFullHeight = 656;
static const int cNormalHeight = 200;
static const int cNormalWidth = 288;
static const int cFullWidth = 480;
//---------------------------------------------------------------------
__fastcall TAddressInputDlg::TAddressInputDlg(TComponent* AOwner)
	: TForm(AOwner), fStart(false), hAsyncTask(NULL)
{
    Grid->DefaultRowHeight = MaskEdit->Height;
    Grid->Cells[0][0] = "MAC";
    Grid->Cells[1][0] = "IP";
    Grid->Cells[2][0] = "Name";
    Height = cNormalHeight;
    Width = cNormalWidth;
    listQueue = new TStringList;
    listQueue->Sorted = false;
}
//---------------------------------------------------------------------
void TAddressInputDlg::SetDefault(TAddressInputDlg::DirectionType dt, AnsiString& as)
{
	switch (dt)
	{
	case DT_BI:
		RadioBtnBi->Checked = true;
		break;
	case DT_SRC:
		RadioBtnSrc->Checked = true;
		break;
	default:
		RadioBtnDst->Checked = true;
		break;
	}
	MaskEdit->Text = as;
}
//---------------------------------------------------------------------
void TAddressInputDlg::InitDlg(bool IP, AnsiString& devName, AnsiString& serviceName)
{
	MaskEdit->SetAddressType(IP);
	IPMode = IP;
    if (IP)
        RadioIP->Checked = true;
    else
        RadioMAC->Checked = true;
    WideString ws(devName);
    try
    {
        if (ws.Length() > 0)
            MacMonitor->DeviceName = ws;
        else
            MacMonitor->DeviceName = WideString(serviceName);
    }
    catch (const Exception& e)
    {
        MacMonitor->DeviceName = WideString(serviceName);
    }
}
//---------------------------------------------------------------------
void __fastcall TAddressInputDlg::FormCloseQuery(TObject *Sender,
	  bool &CanClose)
{
	CanClose = false;
	if (ModalResult == mrOk)
	{
		MaskEdit->CheckAddress();
		ResultString = MaskEdit->GetResultString();
	}
    if (fStart)
    {
        MacMonitor->Stop();
        fStart = false;
    }
    if (hAsyncTask)
    {
        WSACancelAsyncRequest(hAsyncTask);
    }
    CanClose = true;
}
//---------------------------------------------------------------------------
int TAddressInputDlg::GetDirection()
{
	if (RadioBtnBi->Checked)
		return DT_BI;
	if (RadioBtnSrc->Checked)
		return DT_SRC;
	return DT_DST;
}
//---------------------------------------------------------------------------
void __fastcall TAddressInputDlg::RadioAddressClick(TObject *Sender)
{
    bool fNew = (Sender == RadioMAC) ? false : true;
    if (IPMode != fNew)
    {
        IPMode = fNew;
        MaskEdit->Clear();
	    MaskEdit->SetAddressType(IPMode);
    }
    MaskEdit->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TAddressInputDlg::MacMonitorRecvData(TObject *Sender,
      IMacPacket* packet, TOLEBOOL *pStop)
{
    HRESULT hr;
    long lType;
    if (packet->get_IsMulticast()) return;

    BSTR bstrSrcMac = NULL;
    BSTR bstrSrcIP = NULL;
    BSTR bstrDstMac = NULL;
    BSTR bstrDstIP = NULL;
    hr = packet->get_SrcMacString(&bstrSrcMac);
    if (hr == S_OK)
    {
        packet->get_DestMacString(&bstrDstMac);
        packet->get_EtherType(&lType);
        if (lType == ET_IP)
        {
            packet->get_SrcIPAddressString(&bstrSrcIP);
            packet->get_DestIPAddressString(&bstrDstIP);
        }
        CheckAddress(bstrSrcMac, bstrSrcIP);
        CheckAddress(bstrDstMac, bstrDstIP);
        if (bstrDstIP)
        {
            SysFreeString(bstrDstIP);
        }
        if (bstrSrcIP)
        {
            SysFreeString(bstrSrcIP);
        }

        SysFreeString(bstrDstMac);
        SysFreeString(bstrSrcMac);
    }
}
//---------------------------------------------------------------------------
int TAddressInputDlg::GetHashVal(AnsiString& s)
{
    int ret = 0;
    int m = s.Length();
    LPCSTR ps = s.c_str();
    for (int i = 0; i < m; i++, ps++)
    {
        ret << 1;
        ret += (unsigned char)*ps;
    }
    return ret;
}
//---------------------------------------------------------------------------
void TAddressInputDlg::CheckAddress(BSTR mac, BSTR ip)
{
    AnsiString asmac = AnsiString(mac);
    int cHash = GetHashVal(asmac);

    if (iRow >= Grid->RowCount)
        Grid->RowCount = iRow + 5;
    if (ip)
    {
        AnsiString as = AnsiString(ip);
        cHash = GetHashVal(as);
        if (iRow > 1)
        {
            TStrings* ps = Grid->Cols[1];
            for (int i = 1; i <= iRow; i++)
            {
                if (ps->Objects[i] == reinterpret_cast<TObject*>(cHash)
                    && ps->Strings[i].AnsiCompare(as) == 0) return;
            }
        }
        Grid->Cells[1][iRow] = as;
        Grid->Objects[1][iRow] = reinterpret_cast<TObject*>(cHash);
        if (hAsyncTask)
        {
            int n = listQueue->Add(Grid->Cells[1][iRow]);
            listQueue->Objects[n] = reinterpret_cast<TObject*>(iRow);
        }
        else
        {
            reqAddr = inet_addr(as.c_str());
            if (reqAddr)
            {
                currRow = iRow;
                hAsyncTask = WSAAsyncGetHostByAddr(Handle,
                    WM_GOTHOSTNAME,
                    reinterpret_cast<LPCSTR>(&reqAddr),
                    sizeof(ULONG),
                    AF_INET,
                    hostent,
                    MAXGETHOSTSTRUCT);
            }
        }
    }
    else
    {
        if (iRow > 1)
        {
            TStrings* ps = Grid->Cols[0];
            for (int i = 1; i <= iRow; i++)
            {
                if (ps->Objects[i] == reinterpret_cast<TObject*>(cHash)
                    && ps->Strings[i].AnsiCompare(asmac) == 0) return;
            }
        }
    }
    Grid->Cells[0][iRow] = asmac;
    Grid->Objects[0][iRow] = reinterpret_cast<TObject*>(cHash);
    iRow++;
}
//---------------------------------------------------------------------------

void __fastcall TAddressInputDlg::RefButtonClick(TObject *Sender)
{
    if (Height != cFullHeight)
    {
        Height = cFullHeight;
        Width = cFullWidth;
    }
    if (fStart)
    {
        MacMonitor->Stop();
        RefButton->Caption = LoadStr(rsMonStart);
    }
    else
    {
        iRow = 1;
        MacMonitor->Start();
        RefButton->Caption = LoadStr(rsMonStop);
    }
    fStart = !fStart;
}
//---------------------------------------------------------------------------
void __fastcall TAddressInputDlg::MacMonitorStopped(TObject *Sender,
      long ReasonCode)
{
    fStart = false;
    RefButton->Caption = LoadStr(rsMonStart);
}
//---------------------------------------------------------------------------
void __fastcall TAddressInputDlg::GridDblClick(TObject *Sender)
{
    int col = Grid->Col;
    if (Grid->Row > 0 && col >= 0 && col <= 1)
    {
        AnsiString s = Grid->Cells[col][Grid->Row];
        switch (col)
        {
        case 0:
            RadioMAC->Checked = true;
            break;
        case 1:
            RadioIP->Checked = true;
            break;
        }
        MaskEdit->Text = s;
    }
}
//---------------------------------------------------------------------------
void __fastcall TAddressInputDlg::WMGotHostName(TMessage &Message)
{
    if (WSAGETASYNCERROR(Message.LParam) == 0)
    {
        HOSTENT* pent = reinterpret_cast<HOSTENT*>(hostent);
        Grid->Cells[2][currRow] = pent->h_name;
    }
    hAsyncTask = NULL;
    while (listQueue->Count > 0)
    {
        reqAddr = inet_addr(listQueue->Strings[0].c_str());
        currRow = reinterpret_cast<int>(listQueue->Objects[0]);
        listQueue->Delete(0);
        if (reqAddr)
        {
            hAsyncTask = WSAAsyncGetHostByAddr(Handle,
                WM_GOTHOSTNAME,
                reinterpret_cast<LPCSTR>(&reqAddr),
                sizeof(ULONG),
                AF_INET,
                hostent,
                MAXGETHOSTSTRUCT);
            break;
        }
    }
}
//---------------------------------------------------------------------------
__fastcall TAddressInputDlg::~TAddressInputDlg()
{
    MacMonitor->DestroyComponents();
    delete listQueue;
}
//---------------------------------------------------------------------------


