//------------------------------------------------v---------------------
// Copyright(c) 1999 arton
//
#include <vcl.h>
#include <vcl\Registry.hpp>
#pragma hdrstop

#include "About.h"
#include "PacketType.h"
#include "FreePeekMain.h"
#include "FreePeekStrs.h"
#include "AddressSet.h"
#include "IPFilterSetting.h"
#include "MAPI.hpp"
#include "PortAssNum.h"
#include "ip.h"
#include "ValOffset.h"
#include <list>
#include "FilterDoc.h"
#include "DispForm.h"
#include "DescForm.h"
//---------------------------------------------------------------------------
#pragma link "PERFGRAP"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------

__fastcall TMainForm::TMainForm(TComponent *Owner)
	: TForm(Owner), fCapturing(false), lastIndex(-1),
      CaptureEventRecver(NULL), dwCookie(0), ProtoList(NULL), ETypeList(NULL),
      ParamInterval(1000), ParamBuffSize(8), bFilterDirty(false), bCaptureDirty(false),
      pJScript(NULL), SiteObject(this), DispObj(this), bNoCapture(true)
{
    FFileName = LoadStr(sUntitled);
    FFilterFileName = LoadStr(sUntitled);
    Application->OnHint = ShowHint;
    Application->OnIdle = OnIdle;
    sStart = LoadStr(rsStart);
    sStop = LoadStr(rsStop);
    sStartHelp = LoadStr(rsStartHelp);
    sStopHelp = LoadStr(rsStopHelp);
    ScriptIndex = 0;
    TRegistry* reg = new TRegistry;
    if (reg->OpenKey("Software\\arton\\FreePeek", true))
    {
    	try
    	{
    		Left = reg->ReadInteger("FormLeft");
    		Top = reg->ReadInteger("FormTop");
    		ParamInterval = reg->ReadInteger("Interval");
    		ParamBuffSize = reg->ReadInteger("BufferSize");
                ScriptIndex = reg->ReadInteger("ScriptEngine");
        }
    	catch (const Exception &E)
        {
            ;
    	}
        reg->CloseKey();
    }
    delete reg;

    try
    {
        adapters = CoAdapterCollectionEx::Create();
    }
    catch (const Exception &E)
    {
        MessageDlg(LoadStr(rsNoMacCap), mtError, TMsgDlgButtons() << mbOK, 0);
        throw E;
    }
    capEngine = CoMacCapture::Create();
    capEngine->set_MonitorEventInterval(ParamInterval);
    capEngine->set_BufferSize(ParamBuffSize * 1024 * 1024);

    AssPort::FillString(TCPPortList, true);
    AssPort::FillString(UDPPortList, false);

    TLabel* apLab[] = {
        AdapterInfo1, AdapterInfo2, AdapterInfo3,
        AdapterInfo4, AdapterInfo5, AdapterInfo6,
    };
    for (int i = 0; i < COUNTOF(apLab); i++)
    {
        apLab[i]->Caption = "";
    }
    UpDownBuffSize->Position = static_cast<short>(ParamBuffSize);
    UpDownInterval->Position = static_cast<short>(ParamInterval);
    TStringList* list = new TStringList;
    list->Add("JScript");
    list->Add("GlobalRubyScript");
    list->Add("VBScript");
    ComboScrpt->Items = list;
    ComboScrpt->ItemIndex = ScriptIndex;
    PageCtrl->ActivePage = AdapterSheet;

    try
    {
        domDocument = CoDOMDocument::Create();
    }
    catch (const Exception &E)
    {
        MessageDlg(LoadStr(rsNoDOMDoc), mtWarning, TMsgDlgButtons() << mbOK, 0);
    }

	// Command Line Processing
	LPTSTR psz = GetCommandLine();
	if (*psz != _T('"'))
	{
		while (*psz && !isspace(*psz)) psz++;
	}
	else
	{
		psz++;
		while (*psz && *psz != _T('"')) psz++;
		if (*psz == _T('"')) psz++;
	}
	while (*psz && isspace(*psz))
	{
		psz++;
	}
	if (*psz)
	{
		long lResult;
        WideString ws(psz);
	if (capEngine->LoadData(ws, &lResult) == S_OK && lResult == 0)
	{
            FFileName = psz;
            ShowCapture(false);
	}
    }
    //
    CLSID clsid;
    TOleString scrpt(ComboScrpt->Items->Strings[ScriptIndex].c_str());
    if (CLSIDFromProgID(scrpt, &clsid) == S_OK)
    {
        HRESULT hr = CoCreateInstance(clsid, NULL, CLSCTX_SERVER,
            IID_IActiveScript, (void**)&pJScript);
        if (hr != S_OK)
        {
            pJScript = NULL;
        }
        else
        {
            pJScript->SetScriptSite(static_cast<IActiveScriptSite*>(&SiteObject));
            pJScript->AddNamedItem(L"document", SCRIPTITEM_ISVISIBLE | SCRIPTITEM_GLOBALMEMBERS);
            ParseRootScript();
        }
    }
}
//----------------------------------------------------------------------------
void TMainForm::ParseRootScript()
{
    AnsiString as = Application->ExeName;
    static const char* ScrName[] = {
        "FreePeek.js",
        "FreePeek.rb",
        "FreePeek.vbs",
    };
    AnsiString ScrFileName = ScrName[ComboScrpt->ItemIndex];
    int pos = as.LastDelimiter("\\");
    if (pos > 0)
        as = as.SubString(1, pos) + ScrFileName;
    else
        as = ScrFileName;
    IActiveScriptParse* pParse;
    if (pJScript->QueryInterface(IID_IActiveScriptParse, (void**)&pParse) == S_OK)
    {
        try
        {
            TFileStream* str = new TFileStream(as, fmOpenRead | fmShareDenyWrite);
            char* psz = new char[str->Size + 1];
            str->Read(psz, str->Size);
            psz[str->Size] = '\0';
            WideString ws(psz);
            VARIANT var;
            EXCEPINFO excepinfo;
            VariantInit(&var);
            ZeroMemory(&excepinfo, sizeof(EXCEPINFO));
            pParse->ParseScriptText(
                ws.c_bstr(),
                NULL,
                NULL,
                NULL,
                0,
                0,
                SCRIPTTEXT_ISVISIBLE,
                &var,
                &excepinfo);
            delete[] psz;
            delete str;
        }
        catch (const Exception &E)
        {
            ShowMessage(AnsiString(E.ClassName())+ E.Message);
        }
        pParse->Release();
    }
}
//----------------------------------------------------------------------------
void TMainForm::CreateList(const GUID& clsid, TStrings* pList)
{
    IDispatch* pDisp;
    HRESULT hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER,
        IID_IDispatch, (void**)&pDisp);
    if (hr == S_OK)
    {
        pList->Clear();
        DISPPARAMS dispprm = { NULL, NULL, 0, 0 };
        VARIANT vResult;
        VariantInit(&vResult);
        hr = pDisp->Invoke(DISPID_NEWENUM, IID_NULL, LOCALE_USER_DEFAULT,
            DISPATCH_PROPERTYGET, &dispprm, &vResult, NULL, NULL);
        if (hr == S_OK && vResult.vt == VT_UNKNOWN)
        {
            IEnumVARIANT* pEnum;
            if (vResult.punkVal->QueryInterface(IID_IEnumVARIANT, (void**)&pEnum) == S_OK)
            {
                ULONG ul;
		VARIANT v;
		VariantInit(&v);
		while (pEnum->Next(1, &v, &ul) == S_OK)
		{
			if (v.vt == VT_DISPATCH)
			{
				INameValuePair* pNV;
				if (v.pdispVal->QueryInterface(IID_INameValuePair, (void**)&pNV) == S_OK)
				{
                                    char sz[12];
				    BSTR bstr;
				    TVariant vVal;
				    pNV->get_Value(&vVal);
				    pNV->get_Name(&bstr);
                                    wsprintf(sz, " (%d)", static_cast<WORD>(vVal.iVal));
                                    pList->AddObject(OleStrToString(bstr) + sz, reinterpret_cast<TObject*>(vVal.iVal));
                                    VariantClear(&vVal);
                                    SysFreeString(bstr);
                                    pNV->Release();
                                }
                        }
                        VariantClear(&v);
                }
                pEnum->Release();
            }
            VariantClear(&vResult);
        }
        pDisp->Release();
    }
}
//----------------------------------------------------------------------------
void __fastcall TMainForm::ShowHint(TObject *Sender)
{
	StatusBar->Panels->Items[0]->Text = Application->Hint;
}
//----------------------------------------------------------------------------
void __fastcall TMainForm::OnIdle(TObject* Sender, bool& Done)
{
	CoFreeUnusedLibraries();
	Done = true;
}
//----------------------------------------------------------------------------
void __fastcall TMainForm::FileNew1Execute(TObject *Sender)
{
    if (PageCtrl->ActivePage == TabFilter)
    {
        FilterTree->Items->Clear();
    }
    else
    {
        if (PageCtrl->ActivePage != TabCapture)
        {
            PageCtrl->ActivePage = TabCapture;
        }
        CapButtonClick(Sender);
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileOpen1Execute(TObject *Sender)
{
    if (PageCtrl->ActivePage->TabIndex == 1)
    {
        if (!domDocument)
        {
            MessageDlg(LoadStr(rsNoDOMDoc), mtError, TMsgDlgButtons() << mbOK, 0);
            return;
        }
        OpenDialog->Filter = LoadStr(rsFilterExt);
    }
    else
        OpenDialog->Filter = LoadStr(rsCaptureExt);

   	if (OpenDialog->Execute())
   	{
        if (PageCtrl->ActivePage->TabIndex == 1)
        {
            FFilterFileName = OpenDialog->FileName;
            LoadFilter();
        }
        else
        {
            long lResult;
            WideString ws(OpenDialog->FileName);
            HRESULT hr = capEngine->LoadData(ws, &lResult);
            if (hr == S_OK)
            {
                if (lResult == 0)
                {
                    FFileName = OpenDialog->FileName;
                    ShowCapture(false);
                }
                else
                {
                    char sz[128];
                    wsprintf(sz, "File Load Failure %d", lResult);
                    Application->MessageBox(sz, "FreePeek", MB_OK);
                }
            }
        }
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileSave1Execute(TObject *Sender)
{
    AnsiString& filename = (PageCtrl->ActivePage->TabIndex == 1) ? FFilterFileName : FFileName;
    {
    	if (filename == LoadStr(sUntitled))
    		FileSaveAs1Execute(Sender);
    	else
    	{
            if (PageCtrl->ActivePage->TabIndex == 1)
                SaveFilter();
            else
                SaveCapture();
        }
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileSaveAs1Execute(TObject *Sender)
{
	String str;
	TVarRec vrs[1];
    AnsiString& filename = (PageCtrl->ActivePage->TabIndex == 1) ? FFilterFileName : FFileName;

    if (PageCtrl->ActivePage->TabIndex == 1)
    {
        SaveDialog->Filter = LoadStr(rsFilterExt);
        SaveDialog->DefaultExt = "xml";
    }
    else
    {
        SaveDialog->Filter = LoadStr(rsCaptureExt);
        SaveDialog->DefaultExt = "ncp";
    }
    SaveDialog->FileName = filename;
	if (SaveDialog->Execute())
	{
		if (FileExists(SaveDialog->FileName))
		{
			str = FmtLoadStr(sOverwrite, OPENARRAY(TVarRec, (SaveDialog->FileName)));
			if (MessageDlg(str, mtConfirmation, TMsgDlgButtons() << mbYes << mbNo <<
				mbCancel, 0) != IDYES)
				return;
		}
		filename = SaveDialog->FileName;
        if (PageCtrl->ActivePage->TabIndex == 1)
            SaveFilter();
        else
            SaveCapture();
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileSend1Execute(TObject *Sender)
{
	TMapiMessage MapiMessage;
	Cardinal MError;
    AnsiString astr;

    MapiFileDesc fDesc;
    memset(&fDesc, 0, sizeof(MapiFileDesc));
    fDesc.nPosition = -1;
    if (PageCtrl->ActivePage->TabIndex == 1)
    {
        fDesc.lpszPathName = FFilterFileName.c_str();
        astr = FFilterFileName;
    }
    else
    {
        fDesc.lpszPathName = FFileName.c_str();
        astr = FFileName;
    }
    int pos(0);
    for (int i = astr.Length(); i > 1; i--)
        if (astr.IsPathDelimiter(i))
        {
            pos = i;
            break;
        }
    fDesc.lpszFileName = (pos) ? astr.SubString(pos, astr.Length() - pos).c_str() : astr.c_str();

	MapiMessage.ulReserved = 0;
	MapiMessage.lpszSubject = fDesc.lpszFileName;
	MapiMessage.lpszNoteText = NULL;
	MapiMessage.lpszMessageType = NULL;
	MapiMessage.lpszDateReceived = NULL;
	MapiMessage.lpszConversationID = NULL;
	MapiMessage.flFlags = 0;
	MapiMessage.lpOriginator = NULL;
	MapiMessage.nRecipCount = 0;
	MapiMessage.lpRecips = NULL;
	MapiMessage.nFileCount = 1;
	MapiMessage.lpFiles = &fDesc;

	MError = MapiSendMail(0, 0, MapiMessage, MAPI_DIALOG | MAPI_LOGON_UI |
		MAPI_NEW_SESSION, 0);

	if (MError)
		MessageDlg(LoadStr(sSendError), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FileExit1Execute(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::HelpAbout1Execute(TObject *Sender)
{
	Application->CreateForm(__classid(TAboutBox), &AboutBox);
	AboutBox->ShowModal();
	delete AboutBox;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::AdapterSheetShow(TObject *Sender)
{
    if (!adapters) return;
    if (ListAdapters->Items->Count == 0)
    {
        int len = adapters->get_Length();
        for (int i = 0; i < len; i++)
        {
            BSTR bstr;
            bstr = adapters->get_Item(i);
            if (SysStringLen(bstr) <= 0)
            {
                SysFreeString(bstr);
                bstr = NULL;
                bstr = adapters->get_Description(i);
            }
            ListAdapters->Items->Append(bstr);
            SysFreeString(bstr);
        }
    }
    if (ListAdapters->ItemIndex < 0)
    {
        ListAdapters->ItemIndex = 0;
        ListAdaptersClick(Sender);
    }
}
//---------------------------------------------------------------------------
typedef BSTR __fastcall (IAdapterCollectionEx::* strFunc)(long);

void __fastcall TMainForm::ListAdaptersClick(TObject *Sender)
{
    if (!adapters) return;
    int len = adapters->get_Length() - 1;
    int sel = ListAdapters->ItemIndex;
    if (sel >= 0 && sel <= len && sel != lastIndex)
    {
        capEngine->Close();
        TLabel* apLab[] = {
            AdapterInfo2, AdapterInfo3,
            AdapterInfo4, AdapterInfo5, AdapterInfo6,
        };
        strFunc astrFunc[] = {
            &IAdapterCollectionEx::get_AdapterName,
            &IAdapterCollectionEx::get_VendorName,
            &IAdapterCollectionEx::get_Description,
            &IAdapterCollectionEx::get_ServiceName,
            &IAdapterCollectionEx::get_DeviceName,
        };
        for (int i = 0; i < COUNTOF(apLab); i++)
        {
            BSTR bstr = (((IAdapterCollectionEx*)adapters)->*(astrFunc[i]))(sel);
            apLab[i]->Caption = bstr;
            if (i == COUNTOF(apLab) - 1)
            {
                capEngine->Open(bstr);
            }
            else if (i == 0)
            {
                DeviceName = bstr;
            }
            SysFreeString(bstr);
        }

        TVariant v = adapters->get_MACAddress(sel);
        SAFEARRAY* pary = v;
        SafeArrayLock(pary);
        LPBYTE pb = reinterpret_cast<LPBYTE>(pary->pvData);
        char sz[32];
        wsprintfA(sz, "%02X-%02X-%02X-%02X-%02X-%02X",
            *pb, *(pb + 1), *(pb + 2), *(pb + 3), *(pb + 4), *(pb + 5));
        AdapterInfo1->Caption = sz;
        StatusBar->Panels->Items[2]->Text = sz;
        SafeArrayUnlock(pary);
        SafeArrayDestroy(pary);

        lastIndex = sel;
        StatusBar->Panels->Items[1]->Text = AdapterInfo2->Caption;
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::CapButtonClick(TObject *Sender)
{
    if (CaptureEventRecver == NULL && Connect() != S_OK)
    {
        return;
    }

    fCapturing = !fCapturing;
    if (!fCapturing)
    {
        CapButton->Enabled = false;
        ToolButtonNewCap->Enabled = false;
    }
    else
    {
        if (bFilterDirty)
        {
            capEngine->RemoveFilters();
            TTreeNode* p = FilterTree->Items->GetFirstNode();
            if (p)
            {
                IFilterParam* pFilterParam = ConvFilter(p);
                if (pFilterParam)
                {
                    capEngine->AddFilter(pFilterParam);
                    pFilterParam->Release();
                }
            }
            bFilterDirty = false;
        }

        ToolButtonNewCap->Enabled = false;
        ToolButtonOpen->Enabled = false;
        ToolButtonSave->Enabled = false;

        CapButton->Caption = sStop;
        CapButton->Hint = sStopHelp;
        CapButton->SetFocus();

    	for (int i = 0; i < COUNTOF(ATicks); i++) ATicks[i] = 0;
		Bytes = Packets = TicksCount = Ticks = 0;
	    Chart->Series[0]->Clear();	// Packets
    //	Chart->Series[1]->Clear();	// Bytes
	    Chart->BottomAxis->Minimum = 0;
	    Chart->BottomAxis->Maximum = capEngine->get_MonitorEventInterval() * COUNTOF(ATicks);

        StatusBar->Panels->Items[0]->Text = LoadStr(rsCapturing);

        capEngine->Start(VARIANT_TRUE);

        ShowCapture(true);

        ShowHint(Sender);

        CapButton->Caption = sStart;
        CapButton->Hint = sStartHelp;
        CapButton->Enabled = true;

        ToolButtonNewCap->Enabled = FileNew1->Enabled;
        ToolButtonOpen->Enabled = FileOpen1->Enabled;
        ToolButtonSave->Enabled = FileSave1->Enabled;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowCapture(bool fDirty)
{
    IPacketCollectionPtr pColls = capEngine->Packets;
    if (pColls->Length > 0)
    {
        bNoCapture = false;
        bCaptureDirty = fDirty;
        if (fDirty)
            FFileName = LoadStr(sUntitled);
        TCapForm* pForm = new TCapForm(Application);
        pForm->SetFileName(FFileName);
        ProgressBar->Position = 0;
        ProgressBar->Visible = true;
        pForm->SetPackets(pColls, ProgressBar);
        pForm->Show();
        ProgressBar->Visible = false;
    }
}
//---------------------------------------------------------------------------
HRESULT TMainForm::Connect()
{
    CaptureEventRecver = new DMacCaptureEvent(this);
    IConnectionPointContainer* pConnPt;
	HRESULT hr = capEngine->QueryInterface(IID_IConnectionPointContainer, (void**)&pConnPt);
    if (hr == S_OK)
	{
		IConnectionPoint* pConn;
        hr = pConnPt->FindConnectionPoint(DIID__IMacCaptureEvents, &pConn);
    	if (hr == S_OK)
		{
			hr = pConn->Advise(CaptureEventRecver, &dwCookie);
			pConn->Release();
		}
		pConnPt->Release();
	}
    if (hr != S_OK)
    {
        delete CaptureEventRecver;
        CaptureEventRecver = NULL;
    }
    return hr;
}
//---------------------------------------------------------------------------
HRESULT STDMETHODCALLTYPE DMacCaptureEvent::Invoke(
			/* [in] */ DISPID dispIdMember,
			/* [in] */ REFIID riid,
			/* [in] */ LCID lcid,
			/* [in] */ WORD wFlags,
			/* [out][in] */ DISPPARAMS __RPC_FAR *pDispParams,
			/* [out] */ VARIANT __RPC_FAR *pVarResult,
			/* [out] */ EXCEPINFO __RPC_FAR *pExcepInfo,
			/* [out] */ UINT __RPC_FAR *puArgErr)
{
	if (Parent == NULL) return S_OK;
	VARIANTARG* pArg = pDispParams->rgvarg;
	switch (dispIdMember)
	{
	case 1:   // void Stop(long ReasonCode);
		Parent->Stop(pArg->lVal);
		break;
	case 2:	  // void RecvData(IMacPacket* pData, VARIANT_BOOL* pStop);
		// never
		break;
	case 3:	  // void MonitorEvent(long Tick, long Packets, long Bytes, VARIANT_BOOL* pSTop);
		Parent->MonitorEvent((pArg + 3)->lVal, (pArg + 2)->lVal, (pArg + 1)->lVal, pArg->pboolVal);
		break;
	default:
		return DISP_E_MEMBERNOTFOUND;
	}
	return S_OK;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Stop(long ReasonCode)
{
    fCapturing = false;
    CapButton->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MonitorEvent(long aTick, long aPackets, long aBytes, VARIANT_BOOL* pStop)
{
    if (fCapturing == false)
    {
        *pStop = VARIANT_TRUE;
    }

	if ((ULONG)TicksCount >= COUNTOF(ATicks))
	{
		TicksCount = 0;
	}
	if (ATicks[TicksCount])
	{
		Chart->Series[0]->Delete(0);
		Chart->BottomAxis->Maximum = Ticks;
		Chart->BottomAxis->Minimum = ATicks[TicksCount];
	}
	Chart->Series[0]->AddXY(Ticks, aPackets, "", clTeeColor);
//	MonPackForm->Chart->Series[1]->AddXY(Ticks, aBytes, "", clTeeColor);
	Ticks += aTick;
	ATicks[TicksCount] = Ticks;
	TicksCount++;
	Packets += aPackets;
	Bytes += aBytes;
	char sz[64];
	wsprintf(sz, "%d packets, %d bytes captured", Packets, Bytes);
    CaptStat->Caption = sz;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    if (pJScript)
    {
        pJScript->Close();
        pJScript->Release();
    }
    capEngine->Close();
    if (CaptureEventRecver)
    {
        IConnectionPointContainer* pConnPt;
		if (capEngine->QueryInterface(IID_IConnectionPointContainer, (void**)&pConnPt) == S_OK)
		{
			IConnectionPoint* pConn;
			if (pConnPt->FindConnectionPoint(DIID__IMacCaptureEvents, &pConn) == S_OK)
			{
				pConn->Unadvise(dwCookie);
				pConn->Release();
			}
			pConnPt->Release();
		}
		CaptureEventRecver->Parent = NULL;
        delete CaptureEventRecver;
        CaptureEventRecver = NULL;
	}

    domDocument.Unbind();
    capEngine.Unbind();

    if (ProtoList)
        delete ProtoList;
    if (ETypeList)
        delete ETypeList;
    delete UDPPortList;
    delete TCPPortList;

	TRegistry* reg = new TRegistry;
	if (reg->OpenKey("Software\\arton\\FreePeek", true))
	{
		try
		{
			reg->WriteInteger("FormLeft", Left);
			reg->WriteInteger("FormTop", Top);
			reg->WriteInteger("Interval", ParamInterval);
			reg->WriteInteger("BufferSize", ParamBuffSize);
                        reg->WriteInteger("ScriptEngine", ComboScrpt->ItemIndex);
		}
		catch (const Exception &E)
		{
                        ;
		}
		reg->CloseKey();
	}
	delete reg;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::StatusBarResize(TObject *Sender)
{
	int w(0);
	for (int i = 1; i < StatusBar->Panels->Count; i++)
		w += StatusBar->Panels->Items[i]->Width;
	StatusBar->Panels->Items[0]->Width = (StatusBar->Width > w) ? StatusBar->Width - w : 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ButtonCancelClick(TObject *Sender)
{
	UpDownBuffSize->Position = static_cast<short>(ParamBuffSize);
	UpDownInterval->Position = static_cast<short>(ParamInterval);
        ComboScrpt->ItemIndex = ScriptIndex;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ButtonUpdateClick(TObject *Sender)
{
    ParamInterval = UpDownInterval->Position;
    ParamBuffSize = UpDownBuffSize->Position;
    ScriptIndex = ComboScrpt->ItemIndex;
    capEngine->set_MonitorEventInterval(ParamInterval);
    capEngine->set_BufferSize(ParamBuffSize * 1024 * 1024);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FilterTreeDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
    Accept = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PageCtrlChanging(TObject *Sender,
      bool &AllowChange)
{
    AllowChange = !fCapturing;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FilterTreeDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
    TTreeNodes* nodes = FilterTree->Items;
    nodes->BeginUpdate();
    TTreeNode* p = FilterTree->GetNodeAt(X, Y);
    int ix = static_cast<TToolButton*>(Source)->ImageIndex;
    bool done(false);
    if (p == NULL)
    {
        p = nodes->GetFirstNode();
        if (p == NULL)
        {
            p = nodes->Add(NULL, LoadStr(ix));
             done = true;
        }
    }
    if (done == false)
    {
        if (p->ImageIndex >= FBT_AND && p->ImageIndex <= FBT_OR)
        {
            p = nodes->AddChild(p, LoadStr(ix));
        }
        else if (p->ImageIndex == FBT_NOT && p->HasChildren == false)
        {
            p = nodes->AddChild(p, LoadStr(ix));
        }
        else
        {
            TTreeNode* pnew = nodes->AddFirst(p, "");
            p->MoveTo(pnew, naAddChild);
            if (ix >= FBT_AND && ix <= FBT_OR)
            {
                pnew->Text = LoadStr(ix);
                p = pnew;
            }
            else
            {
                pnew->Text = LoadStr(FBT_AND);
                pnew->ImageIndex = FBT_AND;
                pnew->SelectedIndex = FBT_AND;
                p = nodes->AddChild(pnew, LoadStr(ix));
            }
        }
    }
    p->ImageIndex = ix;
    p->SelectedIndex = ix;
    p->Data = reinterpret_cast<void*>(-1);
    FilterTree->FullExpand();
    nodes->EndUpdate();
    bFilterDirty = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::tbfilterDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
    Accept = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FilterTreeMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    TTreeNode* p = FilterTree->GetNodeAt(X, Y);
    if (p == NULL) return;

    FilterTree->Selected = p;
    if (Button == mbRight)
    {
        TPoint pnt(X, Y);
        TPoint pscr = FilterTree->ClientToScreen(pnt);
        FilterPopup->Popup(pscr.x, pscr.y);
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EditActionUpdate(TObject *Sender)
{
    static_cast<TAction*>(Sender)->Enabled = (PageCtrl->ActivePage->TabIndex == 1);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditActionUpdateNOT(TObject *Sender)
{
    static_cast<TAction*>(Sender)->Enabled
        = (PageCtrl->ActivePage->TabIndex == 1
           && FilterTree->Selected != NULL
           && FilterTree->Selected->ImageIndex != FBT_NOT);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditActionUpdateSelNeed(TObject *Sender)
{
    static_cast<TAction*>(Sender)->Enabled
        = (PageCtrl->ActivePage->TabIndex == 1 && FilterTree->Selected != NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditActionUpdateLeaf(TObject *Sender)
{
    static_cast<TAction*>(Sender)->Enabled
        = (PageCtrl->ActivePage->TabIndex == 1 && FilterTree->Selected != NULL
           && FilterTree->Selected->ImageIndex >= FBT_ADDRESS
           && FilterTree->Selected->ImageIndex <= FBT_VALUE);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditActionUpdateAndOr(TObject *Sender)
{
    bool fen(false);
    if (PageCtrl->ActivePage->TabIndex == 1 && FilterTree->Selected != NULL)
    {
        int ix(FilterTree->Selected->ImageIndex);
        if (ix >= FBT_AND && ix <= FBT_OR)
        {
            if (static_cast<TAction*>(Sender)->ImageIndex != ix)
                fen = true;
        }
    }
    static_cast<TAction*>(Sender)->Enabled = fen;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditCopy1Execute(TObject *Sender)
{
    TTreeNode* node = FilterTree->Selected;
    if (!node) node = FilterTree->Items->GetFirstNode();
    if (node)
    {
        TFilterDocument doc;
        doc.Save(node);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditPaste1Execute(TObject *Sender)
{
    TFilterDocument doc(domDocument);
    if (doc.IsValid())
    {
        TTreeNode* node = FilterTree->Selected;
        doc.Load(FilterTree->Items, node);
        bFilterDirty = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditCut1Execute(TObject *Sender)
{
    EditCopy1Execute(Sender);
    EditDelete1Execute(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditDelete1Execute(TObject *Sender)
{
    TTreeNode* node = FilterTree->Selected;
    if (node)
    {
        FilterTree->Items->BeginUpdate();
        if (node->ImageIndex == FBT_NOT && node->HasChildren)
        {
            node->getFirstChild()->MoveTo(node, naInsert);
        }
        node->Delete();
        FilterTree->FullExpand();
        FilterTree->Items->EndUpdate();
        bFilterDirty = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddNotExecute(TObject *Sender)
{
    TTreeNode* p = FilterTree->Selected;
    if (p && p->ImageIndex != FBT_NOT)
    {
        AnsiString caption = LoadStr(FBT_NOT);
        TTreeNodes* nodes = FilterTree->Items;
        nodes->BeginUpdate();
        TTreeNode* pnew = (p->Index) ? nodes->Add(p, caption) : nodes->AddFirst(p, caption);
        p->MoveTo(pnew, naAddChild);
        pnew->ImageIndex = FBT_NOT;
        pnew->SelectedIndex = FBT_NOT;
        FilterTree->FullExpand();
        nodes->EndUpdate();
        bFilterDirty = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddAndOrExecute(TObject *Sender)
{
    TTreeNode* p = FilterTree->Selected;
    if (p)
    {
        int newindex(0);
        if (p->ImageIndex == FBT_OR)
        {
            newindex = FBT_AND;
        }
        else if (p->ImageIndex == FBT_AND)
        {
            newindex = FBT_OR;
        }
        if (newindex)
        {
            p->Text = LoadStr(newindex);
            p->ImageIndex = newindex;
            p->SelectedIndex = newindex;
            FilterTree->Repaint();
        }
        bFilterDirty = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EditOptionExecute(TObject *Sender)
{
    TTreeNode* p = FilterTree->Selected;
    if (p)
    {
        switch (p->ImageIndex)
        {
        case FBT_ETYPE:
            Application->CreateForm(__classid(TPacketTypeDlg), &PacketTypeDlg);
            PacketTypeDlg->Caption = LoadStr(rsPacketDlgTitle);
            if (ETypeList == NULL)
            {
                ETypeList = new TStringList;
                CreateList(CLSID_EtherTypeNameCollection, ETypeList);
            }
            PacketTypeDlg->List->Items = ETypeList;
            PosForm(PacketTypeDlg);
            if (PacketTypeDlg->ShowModal() == IDOK)
            {
                p->Text = PacketTypeDlg->GetResultString();
                p->Data = reinterpret_cast<void*>(PacketTypeDlg->GetResultValue());
            }
            delete PacketTypeDlg;
            break;
        case FBT_ADDRESS:
            Application->CreateForm(__classid(TAddressInputDlg), &AddressInputDlg);
            PosForm(AddressInputDlg);
            AddressInputDlg->InitDlg(true, DeviceName, AdapterInfo5->Caption);
            if ((long)p->Data > 0)
            {
                long l = (long)p->Data;
                int ix = p->Text.AnsiPos(":");
                if (ix > 0)
                    AddressInputDlg->SetDefault((BYTE)(l >> 8), p->Text.c_str() + ix);
            }
            if (AddressInputDlg->ShowModal() == IDOK)
            {
                int dir = AddressInputDlg->GetDirection();
                AnsiString s = LoadStr(sSenderStr + dir);
                s += ":" + AddressInputDlg->GetResult();
                p->Text = s;
                p->Data = reinterpret_cast<void*>(MAKELONG(MAKEWORD(AddressInputDlg->GetType(), dir), 0));
            }
            delete AddressInputDlg;
            break;
        case FBT_IP:
            Application->CreateForm(__classid(TIPFilterDlg), &IPFilterDlg);
            if (ProtoList == NULL)
            {
                ProtoList = new TStringList;
                CreateList(CLSID_ProtocolNameCollection, ProtoList);
            }
            IPFilterDlg->SetProtocolList(ProtoList);
            IPFilterDlg->SetTCPList(TCPPortList);
            IPFilterDlg->SetUDPList(UDPPortList);
            IPFilterDlg->ComboBox->Items = ProtoList;
            PosForm(IPFilterDlg);
            if (IPFilterDlg->ShowModal() == IDOK)
            {
                int dir = IPFilterDlg->GetDirection();
                p->Text = IPFilterDlg->GetResultString();
                int data = MAKELONG(MAKEWORD(IPFilterDlg->GetFilterType(), dir), IPFilterDlg->GetResult());
                p->Data = reinterpret_cast<void*>(data);
            }
            delete IPFilterDlg;
            break;
        case FBT_VALUE:
            Application->CreateForm(__classid(TValOffsetDlg), &ValOffsetDlg);
            PosForm(ValOffsetDlg);
            if (ValOffsetDlg->ShowModal() == IDOK)
            {
                p->Text = IntToStr(ValOffsetDlg->GetOffset()) + ":" + ValOffsetDlg->GetData();
                p->Data = NULL;
            }
            delete ValOffsetDlg;
            break;
        default:
            break;
        }
        bFilterDirty = true;
    }
}
//---------------------------------------------------------------------------
void TMainForm::PosForm(TForm* pform)
{
    TPoint pnt = Mouse->CursorPos;
    pform->Top = pnt.y;
    pform->Left = pnt.x;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileActionUpdate(TObject *Sender)
{
    TAction* pAction = static_cast<TAction*>(Sender);
    if (fCapturing)
    {
        pAction->Enabled = false;
        return;
    }
    bool fen(true);
    int newIndex(-1);
    if (PageCtrl->ActivePage->TabIndex == 1)
    {
        switch (pAction->Tag)
        {
        case FAT_NEW:
            pAction->Hint = LoadStr(rsNewHelp);
            newIndex = 6;
            break;
        case FAT_SAVE:
        case FAT_SAVEAS:
            if (FilterTree->Items->Count <= 0)
                fen = false;
            break;
        case FAT_SEND:
            if (FFilterFileName == LoadStr(sUntitled))
                fen = false;
            break;
        default:
            break;
        }
    }
    else
    {
        switch (pAction->Tag)
        {
        case FAT_NEW:
            pAction->Hint = LoadStr(rsStartHelp);
            newIndex = 16;
            break;
        case FAT_SAVE:
            if (bCaptureDirty == false)
            {
                fen = false;
            }
            else
            {
                try
                {
                    if (capEngine->Packets->Length <= 0)
                        fen = false;
                }
                catch (const Exception& e)
                {
                    ;
                }
            }
            break;
        case FAT_SAVEAS:
            if (bNoCapture)
                fen = false;
            break;
        case FAT_SEND:
            if (bNoCapture || FFileName == LoadStr(sUntitled))
                fen = false;
            break;
        default:
            break;
        }
    }
    pAction->Enabled = fen;
    if (newIndex >= 0)
    {
        pAction->ImageIndex = newIndex;
    }
}
//---------------------------------------------------------------------------
void TMainForm::SaveFilter()
{
    try
    {
        TFilterDocument fdoc(FFilterFileName);
        fdoc.Save(FilterTree->Items->GetFirstNode());
    }
    catch (const Exception &E)
    {
        ShowMessage(AnsiString(E.ClassName())+ E.Message);
    }
}
//---------------------------------------------------------------------------
void TMainForm::LoadFilter()
{
    try
    {
        TFilterDocument fdoc(FFilterFileName, domDocument);
        fdoc.Load(FilterTree->Items);
        bFilterDirty = true;
    }
    catch (const Exception &E)
    {
        ShowMessage(AnsiString(E.ClassName())+ E.Message);
    }
}
//---------------------------------------------------------------------------
void TMainForm::SaveCapture()
{
    int nExt = FFileName.LastDelimiter(".");
    AnsiString ext("ncp");
    if (nExt == 0)
    {
        FFileName = FFileName + "." + ext;
    }
    else
    {
        nExt++;
        ext = FFileName.SubString(nExt, FFileName.Length() - nExt + 1);
    }
    if (ext.AnsiCompareIC("ncp") == 0)
    {
        ;
    }
    else
    {
        FFileName = FFileName + ".ncp";
    }
    WideString ws(FFileName);
    long lResult = capEngine->SaveData(ws);
    bCaptureDirty = false;
}
//---------------------------------------------------------------------------
void TMainForm::SetDirection(Maccapv2lib_tlb::IFilterParam* pFilter, int dir)
{
    switch (dir)
    {
    case 2:
        pFilter->set_Source(VARIANT_TRUE);
        // fall down to the next case
    case 1:
        pFilter->set_Destination(VARIANT_TRUE);
        break;
    case 0:
        pFilter->set_Source(VARIANT_TRUE);
        break;
    }
}
//---------------------------------------------------------------------------
Maccapv2lib_tlb::IFilterParam* TMainForm::ConvFilter(TTreeNode* pNode)
{
    std::list<IFilterParam*> listFilters;
    IFilterParam* pFilter = NULL;
    int val = reinterpret_cast<int>(pNode->Data);
    AnsiString& as = pNode->Text;
    VARIANT v;
    VariantInit(&v);
    switch (pNode->ImageIndex)
    {
    case FBT_ADDRESS:
        if (val < 0) return NULL;
        {
        CoCreateInstance(CLSID_FilterParam, NULL, CLSCTX_INPROC_SERVER,
            IID_IFilterParam, (void**)&pFilter);
        pFilter->set_Type(static_cast<Maccapv2lib_tlb::FilterTypeConstants>(LOBYTE(LOWORD(val))));
        SetDirection(pFilter, HIBYTE(LOWORD(val)));
        int pos = as.Pos(":");
        v.vt = VT_BSTR;
        AnsiString test = as.SubString(pos + 1, as.Length() - pos).c_str();
        v.bstrVal = AnsiToOLESTR(as.SubString(pos + 1, as.Length() - pos).c_str());
        pFilter->set_Value(v);
        }
        break;
    case FBT_ETYPE:
        if (val < 0) return NULL;
        CoCreateInstance(CLSID_FilterParam, NULL, CLSCTX_INPROC_SERVER,
            IID_IFilterParam, (void**)&pFilter);
        pFilter->set_Type(FT_ETHERTYPE);
        v.vt = VT_I4;
        v.lVal = val;
        pFilter->set_Value(v);
        break;
    case FBT_IP:
        if (val < 0) return NULL;
        CoCreateInstance(CLSID_FilterParam, NULL, CLSCTX_INPROC_SERVER,
            IID_IFilterParam, (void**)&pFilter);
        pFilter->set_Type(static_cast<Maccapv2lib_tlb::FilterTypeConstants>(LOBYTE(LOWORD(val))));
        SetDirection(pFilter, HIBYTE(LOWORD(val)));
        v.vt = VT_I4;
        v.lVal = HIWORD(val);
        pFilter->set_Value(v);
        break;
    case FBT_VALUE:
        if (val < 0) return NULL;
        {
        CoCreateInstance(CLSID_FilterParam, NULL, CLSCTX_INPROC_SERVER,
            IID_IFilterParam, (void**)&pFilter);
        int pos = as.Pos(":");
        SAFEARRAYBOUND rgsabound[1];
        rgsabound[0].lLbound = 0;
        rgsabound[0].cElements = 2;
        v.vt = VT_ARRAY | VT_VARIANT;
        v.parray = SafeArrayCreate(VT_VARIANT, 1, rgsabound);
        SafeArrayLock(v.parray);
        VARIANT* pvar = reinterpret_cast<VARIANT*>(v.parray->pvData);
        pvar->vt = VT_I2;
        pvar->iVal = static_cast<short>(StrToInt(as.SubString(1, pos - 1)));
        pvar++;
        pvar->vt = VT_BSTR;
        pvar->bstrVal = AnsiToOLESTR(as.SubString(pos + 1, as.Length() - pos).c_str());
        SafeArrayUnlock(v.parray);
        pFilter->set_Type(FT_PACKETOFFSETVALUE);
        pFilter->set_Value(v);
        }
        break;
    case FBT_AND:
    case FBT_OR:
        if (pNode->HasChildren)
        {
            TTreeNode* pChild = pNode->getFirstChild();
            while (pChild)
            {
                IFilterParam* p = ConvFilter(pChild);
                if (p)
                    listFilters.push_back(p);
                pChild = pNode->GetNextChild(pChild);
            }
            if (listFilters.size() > 0)
            {
                CoCreateInstance(CLSID_FilterParam, NULL, CLSCTX_INPROC_SERVER,
                    IID_IFilterParam, (void**)&pFilter);
                SAFEARRAYBOUND rgsabound[1];
                rgsabound[0].lLbound = 0;
                rgsabound[0].cElements = listFilters.size();
                v.vt = VT_ARRAY | VT_DISPATCH;
                v.parray = SafeArrayCreate(VT_DISPATCH, 1, rgsabound);
                SafeArrayLock(v.parray);
                IFilterParam** p = reinterpret_cast<IFilterParam**>(v.parray->pvData);
                for (std::list<IFilterParam*>::iterator it = listFilters.begin(); it != listFilters.end(); it++)
                {
                    *p++ = *it;
                }
                SafeArrayUnlock(v.parray);
                pFilter->set_Type(static_cast<_FilterTypeConstants>(FT_JOINT_AND + (pNode->ImageIndex - FBT_AND)));
                pFilter->set_Value(v);
            }
        }
        break;
    case FBT_NOT:
        if (pNode->HasChildren)
        {
            TTreeNode* pChild = pNode->getFirstChild();
            IFilterParam* p = ConvFilter(pChild);
            if (p)
            {
                CoCreateInstance(CLSID_FilterParam, NULL, CLSCTX_INPROC_SERVER,
                    IID_IFilterParam, (void**)&pFilter);
                v.vt = VT_DISPATCH;
                v.pdispVal = p;
                pFilter->set_Type(static_cast<_FilterTypeConstants>(FT_NOT));
                pFilter->set_Value(v);
            }
        }
        break;
    }
    VariantClear(&v);

    return pFilter;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ShowCapture1Execute(TObject *Sender)
{
    ShowCapture(false);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowCapture1Update(TObject *Sender)
{
    bool fen(false);
    TAction* pAction = static_cast<TAction*>(Sender);
    IPacketCollectionPtr pColls;
    if (capEngine->get_Packets((Maccapv2lib_tlb::IPacketCollectionPtr*)&pColls) == S_OK && pColls->Length > 0)
    {
        fen = true;
    }
    pAction->Enabled = fen;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormActivate(TObject *Sender)
{
	Application->OnHint = ShowHint;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDeactivate(TObject *Sender)
{
	StatusBar->Panels->Items[0]->Text = "";
}
//---------------------------------------------------------------------------
STDMETHODIMP TMainForm::SiteObj::OnScriptError(IActiveScriptError* pase)
{
    AnsiString asSourceLine;
	BSTR pstrErr = NULL;
	pase->GetSourceLineText(&pstrErr);
	if (pstrErr)
	{
        asSourceLine = pstrErr;
		SysFreeString(pstrErr);
	}
	else
	{
        asSourceLine = "no source line";
	}
    long lErrorLine, lErrorColumn;
	DWORD dwContext;
	pase->GetSourcePosition(&dwContext, (unsigned long*)&lErrorLine, &lErrorColumn);
	EXCEPINFO excep;
	memset(&excep, 0, sizeof(EXCEPINFO));
	pase->GetExceptionInfo(&excep);
    AnsiString asDesc;
	if (excep.bstrDescription)
	{
        asDesc = excep.bstrDescription;
		SysFreeString(excep.bstrDescription);
	}

    AnsiString as = "Script Err: line=" + IntToStr((int)lErrorLine);
    as += ", columne=" + IntToStr((int)lErrorColumn);
    as += "\r\nDescription: " + asDesc;
    as += "\r\nSource: " + asSourceLine;
    MessageDlg(as, mtWarning, TMsgDlgButtons() << mbOK, 0);
    
	return S_OK;
}
//---------------------------------------------------------------------------
void TMainForm::SiteObj::ShowPacket(IMacPacket* pPacket)
{
    pOuter->pJScript->SetScriptState(SCRIPTSTATE_CONNECTED);
    IDispatch* pDisp;
	HRESULT hr = pOuter->pJScript->GetScriptDispatch(NULL, &pDisp);
    if (hr == S_OK)
    {
    	DISPID dispid;
        static LPOLESTR func[] = { L"show_packet" };
	    hr = pDisp->GetIDsOfNames(IID_NULL, func, 1, LOCALE_SYSTEM_DEFAULT, &dispid);
        if (hr == S_OK)
        {
            VARIANT v, vRetVal;
            VariantInit(&vRetVal);
            v.vt = VT_DISPATCH;
            v.pdispVal = pPacket;
        	UINT uiArgErr(0);
        	DISPPARAMS dispprm;
        	memset(&dispprm, 0, sizeof(DISPPARAMS));
            dispprm.rgvarg = &v;
            dispprm.cArgs = 1;
            pDisp->Invoke(dispid, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD,
                &dispprm, &vRetVal, NULL, &uiArgErr);
            VariantClear(&vRetVal);
        }
        pDisp->Release();
    }
    pOuter->pJScript->SetScriptState(SCRIPTSTATE_DISCONNECTED);
}
//---------------------------------------------------------------------------
STDMETHODIMP TMainForm::Dispatch::GetIDsOfNames(
          REFIID riid,
          LPOLESTR* rgszNames,
          unsigned int cNames,
          LCID lcid,
          DISPID FAR* rgDispId)
{
    const static LPCOLESTR ap[] = {
        L"write",
        L"writeln",
        L"alert",
        L"dump",
        L"color",
        L"size",
        L"bold",
        L"italic",
        L"tcpport",
        L"udpport",
        L"icmptype",
    };
    for (int i = 0; i < COUNTOF(ap); i++)
    {
        if (_wcsicmp(*rgszNames, ap[i]) == 0)
        {
            *rgDispId = i + 1;
            return S_OK;
        }
    }
    return DISP_E_MEMBERNOTFOUND;
}
//---------------------------------------------------------------------------
STDMETHODIMP TMainForm::Dispatch::Invoke(
          DISPID dispIdMember,
          REFIID riid,
          LCID lcid,
          WORD wFlags,
          DISPPARAMS FAR* pDispParams,
          VARIANT FAR* pVarResult,
          EXCEPINFO FAR* pExcepInfo,
          unsigned int FAR* puArgErr)
{
    HRESULT hr;
    VARIANT v;
    VariantInit(&v);
    switch (dispIdMember)
    {
    case DID_WRITE:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_BSTR);
        if (hr != S_OK)
            return hr;
        Write(v.bstrVal);
        VariantClear(&v);
        break;
    case DID_WRITELINE:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_BSTR);
        if (hr != S_OK)
            return hr;
        WriteLine(v.bstrVal);
        VariantClear(&v);
        break;
    case DID_ALERT:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_BSTR);
        if (hr != S_OK)
            return hr;
        Alert(v.bstrVal);
        VariantClear(&v);
        break;
    case DID_DUMP:
        if (pDispParams->cArgs != 2)
            return DISP_E_BADPARAMCOUNT;
        return Dump(pDispParams);
    case DID_COLOR:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_I4);
        if (hr != S_OK) return hr;
        Color(v.lVal);
        break;
    case DID_SIZE:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_I4);
        if (hr != S_OK) return hr;
        Size(v.lVal);
        break;
    case DID_BOLD:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_BOOL);
        if (hr != S_OK) return hr;
        Bold((v.boolVal) ? true : false);
        break;
    case DID_ITALIC:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_BOOL);
        if (hr != S_OK) return hr;
        Italic((v.boolVal) ? true : false);
        break;
    case DID_TCPPORT:
    case DID_UDPPORT:
    case DID_ICMPTYPE:
        if (pDispParams->cArgs != 1)
            return DISP_E_BADPARAMCOUNT;
        hr = VariantChangeType(&v, pDispParams->rgvarg, 0, VT_I4);
        if (hr != S_OK) return hr;
        VariantInit(pVarResult);
        pVarResult->vt = VT_BSTR;
        if (dispIdMember == DID_TCPPORT)
            pVarResult->bstrVal = StringToOleStr(TcpPort(v.lVal));
        else if (dispIdMember == DID_UDPPORT)
            pVarResult->bstrVal = StringToOleStr(UdpPort(v.lVal));
        else
            pVarResult->bstrVal = StringToOleStr(ICMPType(v.lVal));
        break;
    default:
        return DISP_E_MEMBERNOTFOUND;
    }
    return S_OK;
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::Write(AnsiString s)
{
    pDescForm->DataView->SelText = s;
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::WriteLine(AnsiString s)
{
    pDescForm->DataView->Lines->Append(s);
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::Alert(AnsiString s)
{
    Application->MessageBox(s.c_str(), "FreePeek", MB_OK);
}
//---------------------------------------------------------------------------
HRESULT TMainForm::Dispatch::Dump(DISPPARAMS FAR* pDispParams)
{
    VARTYPE avt[] = {
        VT_I4, VT_BSTR
    };
    VARIANT av[2];
    HRESULT hr;
    for (int i = 0; i < 2; i++)
    {
        VariantInit(&av[i]);
        hr = VariantChangeType(&av[i], pDispParams->rgvarg + i, 0, avt[i]);
        if (hr != S_OK) return hr;
    }
    AnsiString as(av[1].bstrVal);
    pDescForm->DumpData(as.c_str(), av[0].lVal);
    for (int i = 0; i < 2; i++)
        VariantClear(&av[i]);
    return S_OK;
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::Color(long l)
{
    pDescForm->DataView->SelAttributes->Color = l;
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::Size(long l)
{
    pDescForm->DataView->SelAttributes->Size = l;
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::Bold(bool f)
{
    TFontStyles tf = pDescForm->DataView->SelAttributes->Style;
    pDescForm->DataView->SelAttributes->Style = (f) ? (tf << fsBold) : (tf >> fsBold);
}
//---------------------------------------------------------------------------
void TMainForm::Dispatch::Italic(bool f)
{
    TFontStyles tf = pDescForm->DataView->SelAttributes->Style;
    pDescForm->DataView->SelAttributes->Style = (f) ? (tf << fsItalic) : (tf >> fsItalic);
}
//---------------------------------------------------------------------------
AnsiString TMainForm::Dispatch::TcpPort(long l)
{
    LPCSTR p = AssPort::SearchTCPPort(l);
    if (p == NULL)
        p = "";
    return p;
}
//---------------------------------------------------------------------------
AnsiString TMainForm::Dispatch::UdpPort(long l)
{
    LPCSTR p = AssPort::SearchUDPPort(l);
    if (p == NULL)
        p = "";
    return p;
}
//---------------------------------------------------------------------------
AnsiString TMainForm::Dispatch::ICMPType(long l)
{
    return GetICMPTypeName((BYTE)l);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ReparseScriptUpdate(TObject *Sender)
{
    static_cast<TAction*>(Sender)->Enabled = ScriptAvail();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ReparseScriptExecute(TObject *Sender)
{
    ParseRootScript();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject *Sender, bool &CanClose)
{
    if (fCapturing)
    {
        CapButtonClick(Sender);
        CanClose = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileExit1Update(TObject *Sender)
{
    static_cast<TAction*>(Sender)->Enabled = !fCapturing;
}
//---------------------------------------------------------------------------



