// Copyright(c) 1999 arton

#include "stdafx.h"
#include "MacCapv2.h"
#include "MacMonitor.h"
#include "Adapters.h"
#include "MacPacket.h"

/////////////////////////////////////////////////////////////////////////////
// CMacMonitor
HRESULT CMacMonitor::FinalConstruct()
{
	IAdapterCollectionEx* pAdapter;
	IClassFactory* pFac;
	HRESULT hr = _Module.GetClassObject(CLSID_AdapterCollectionEx, IID_IClassFactory, (void**)&pFac);
	if (hr != S_OK) return hr;
	hr = pFac->CreateInstance(NULL, IID_IAdapterCollectionEx, (void**)&pAdapter);
	pFac->Release();
	if (hr == S_OK)
	{
		m_pAdapters = static_cast<CAdapters*>(pAdapter);
		m_szDevice = m_pAdapters->GetAdapterInfo()[0].m_sProductName;
		m_bValid = TRUE;
	}

	m_hThreadSync = CreateEvent(NULL, TRUE, TRUE, NULL);
	return S_OK;
}

void CMacMonitor::FinalRelease() 
{
	Clear();
	if (m_hThread)
	{
		DWORD dw;
		GetExitCodeThread(m_hThread, &dw);
		if (dw == STILL_ACTIVE)
		{
			ResetEvent();
			PostThreadMessage(m_idThread, WM_QUIT, 0, 0);
			WaitForSingleObject(m_hThreadSync, INFINITE);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	if (m_pAdapters)
		m_pAdapters->Release();
	for (AFilterIter it = m_afilter.begin(); it != m_afilter.end(); it++)
		delete (*it);
	m_afilter.clear();
	CloseHandle(m_hThreadSync);
	if (m_hWnd)
		DestroyWindow();
}

HRESULT CMacMonitor::CreateEventWindow()
{
	if (!m_hWndCD)
	{
		RECT rect;
		rect.top = rect.bottom = rect.left = rect.right = 0;
		CComPtr<IOleWindow> pWnd;
		if (m_spInPlaceSite)
			m_spInPlaceSite->QueryInterface(IID_IOleWindow, (void**)&pWnd);
		else if (m_spClientSite)
			m_spClientSite->QueryInterface(IID_IOleWindow, (void**)&pWnd);
		if (pWnd)
		{
			HWND hwnd;
			if (pWnd->GetWindow(&hwnd) == S_OK)
			{
				m_hWndCD = CreateControlWindow(hwnd, rect);
			}
			pWnd.Release();
		}
		else
		{
			if (Create(NULL, rect, NULL, WS_POPUP) == NULL)
				return Error(IDS_E_INVALIDCONTAINER);
		}
	}
	return S_OK;
}

unsigned long WINAPI CMacMonitor::RecvThreadProc(LPVOID lpParameter)
{
	CoInitialize(NULL);

	CMacMonitor* pMonitor = reinterpret_cast<CMacMonitor*>(lpParameter);

	int iCurrent(0);
	OVERLAPPED ov[MAX_IO];
	LPBYTE abuff[MAX_IO];
	std::list<LPBYTE> listFree;
	for (int i = 0; i < MAX_IO; i++)
		listFree.push_back(new BYTE[BUFF_SIZE]);
	for (i = 0; i < MAX_IO; i++)
	{
		abuff[i] = new BYTE[BUFF_SIZE];
		ZeroMemory(&ov[i], sizeof(OVERLAPPED));
		ov[i].hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	}
	HANDLE hAdapter(INVALID_HANDLE_VALUE);
	MSG msg;
	PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE);	// Create Queue
	pMonitor->SetEvent(S_OK);
	bool fStart(false);

	DWORD dw;
	for (;;)
	{
		if (!fStart)
		{
			if (::GetMessage(&msg, NULL, 0, 0) <= 0) break;
			dw = WAIT_OBJECT_0 + 1;
		}
		else
		{
			if (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			{
				if (msg.message == WM_QUIT) break;
				dw = WAIT_OBJECT_0 + 1;
			}
			else
			{
				dw = MsgWaitForMultipleObjects(1, &ov[iCurrent].hEvent, FALSE, INFINITE, QS_ALLINPUT);
				if (dw == WAIT_OBJECT_0 + 1)
					continue;
			}
		}
		
		if (dw == WAIT_OBJECT_0)
		{
			LPBYTE pbRead = abuff[iCurrent];
			BOOL Result = GetOverlappedResult(hAdapter, &ov[iCurrent], &dw, TRUE);
			if (listFree.empty())
			{
				abuff[iCurrent] = new BYTE[BUFF_SIZE];
			}
			else
			{
				abuff[iCurrent] = listFree.front();
				listFree.pop_front();
			}
			BOOL f = ReadFile(hAdapter, abuff[iCurrent], BUFF_SIZE, NULL, &ov[iCurrent]);
			if (!f)
			{
				ATLTRACE(_T("Read Failed %d\n"), GetLastError());
			}
			iCurrent++;
			if (iCurrent == MAX_IO)
				iCurrent = 0;
			if (Result && (pMonitor->m_fRecvDataEnabled || pMonitor->m_lMonitorEventInterval))
			{
				bool fSkip(false);
				if (pMonitor->m_afilter.empty() == false)
				{
					fSkip = true;
					for (AFilterIter it = pMonitor->m_afilter.begin(); it != pMonitor->m_afilter.end(); it++)
					{
						if ((*it)->CheckRange(dw) && (*it)->DataCompare(dw, pbRead))
						{
							fSkip = false;
							break;
						}
					}
				}
				if (fSkip == false)
				{
					pMonitor->AddTally(dw);
					if (pMonitor->m_fRecvDataEnabled)
					{
						pMonitor->PostMessage(pMonitor->m_uiReadBuffer, (WPARAM)dw, (LPARAM)pbRead);
						pbRead = NULL;	// maybe collect by pMonitor->FreeRequest later
					}
				}
			}
			if (pbRead)
				listFree.push_front(pbRead);
		}
		else if (dw == WAIT_OBJECT_0 + 1)
		{
			if (pMonitor->IsStartRequest(msg.message))
			{
				hAdapter = pMonitor->OpenAdapter();
				if (hAdapter == INVALID_HANDLE_VALUE)
				{
					pMonitor->SetEvent(MAKE_HRESULT(SEVERITY_ERROR, FACILITY_WIN32, GetLastError()));
				}
				else
				{
					for (i = 0; i < MAX_IO; i++)
					{
						BOOL f = ReadFile(hAdapter, abuff[i], BUFF_SIZE, NULL, &ov[i]);
						if (!f)
						{
							ATLTRACE(_T("Read Failed %d\n"), GetLastError());
						}
					}
					iCurrent = 0;
					fStart = true;
					pMonitor->SetEvent(S_OK);
				}
			}
			else if (pMonitor->IsStopRequest(msg.message))
			{
				if (hAdapter != INVALID_HANDLE_VALUE)
				{
					CancelIo(hAdapter);
					CloseHandle(hAdapter);
					hAdapter = INVALID_HANDLE_VALUE;
				}
				pMonitor->SetEvent(S_OK);
				fStart = false;
			}
			else if (pMonitor->IsFreeRequest(msg.message))
			{
				listFree.push_front(reinterpret_cast<LPBYTE>(msg.lParam));
			}
			::DispatchMessage(&msg);
		}
	}

	if (hAdapter != INVALID_HANDLE_VALUE)
	{
		CancelIo(hAdapter);
		CloseHandle(hAdapter);
	}
	for (i = 0; i < MAX_IO; i++)
		CloseHandle(ov[i].hEvent);

	for (std::list<LPBYTE>::iterator it = listFree.begin(); it != listFree.end(); it++)
	{
		delete[] *it;
	}
	pMonitor->SetEvent(S_OK);
	CoUninitialize();

	return 0;
}

LRESULT CMacMonitor::OnReadPakcet(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	LPBYTE pbPacket = reinterpret_cast<LPBYTE>(lParam);
	CComObject<CMacPacket>* p;
	HRESULT hr = CComObject<CMacPacket>::CreateInstance(&p);
	if (hr == S_OK)
	{
		VARIANT_BOOL vStop(VARIANT_FALSE);
		p->InternalAddRef();
		p->SetParam(wParam, pbPacket);
		PostThreadMessage(m_idThread, m_uiFreeBuffer, 0, lParam);
		if (m_bEnabled)
		{
			Fire_RecvData(p, &vStop);
			p->Release();
			if (vStop)
			{
				Stop();
			}
		}
		else
		{
			m_listPackets.push_back(p);
		}
	}
	else
		PostThreadMessage(m_idThread, m_uiFreeBuffer, 0, lParam);

	bHandled = TRUE;
	return TRUE;
}

LRESULT CMacMonitor::OnTimer(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if (m_bEnabled)
	{
		VARIANT_BOOL vStop(VARIANT_FALSE);
		DWORD dw = GetTickCount();
		Fire_MonitorEvent(dw - m_lLastTick, m_lPackets, m_lBytes, &vStop);
		if (vStop)
			Stop();
		else
			m_lLastTick = dw;
	}
	return FALSE;
}

LRESULT CMacMonitor::OnFireNext(UINT, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if (m_fMonitoring && m_bEnabled && m_listPackets.size() > 0)
	{
		IMacPacket* p = m_listPackets.front();
		m_listPackets.pop_front();
		VARIANT_BOOL vStop(VARIANT_FALSE);
		Fire_RecvData(p, &vStop);
		p->Release();
		if (vStop)
		{
			Stop();
		}
	}
	return TRUE;
}

HRESULT CMacMonitor::SendOnDataChange(DWORD advf)
{
	if (m_fMonitoring && m_bEnabled && m_listPackets.size() > 0)
	{
		PostMessage(m_uiFireNext);
	}
	return CComControlBase::SendOnDataChange(advf);
}


//////////////////////////////////////////////////////////////////////////////////////
#if 0
HRESULT STDMETHODCALLTYPE CMacMonitor::GetPredefinedStrings( DISPID dispID, CALPOLESTR* pCaStringsOut, CADWORD* pCaCookiesOut )
{
	if (!pCaStringsOut && !pCaCookiesOut) return E_POINTER;

	static const LPOLESTR aszBool[] = { L"True", L"False", };

	switch (dispID)
	{
	case dispidDeviceName:
	{
		AdapterArray& array = m_pAdapters->GetAdapterInfo();
		int cb = array.size();
		if (cb == 0) return E_NOTIMPL;

		pCaCookiesOut->cElems = pCaStringsOut->cElems = cb;
		pCaStringsOut->pElems = reinterpret_cast<LPOLESTR*>(CoTaskMemAlloc(sizeof(LPOLESTR) * array.size()));
		if (pCaStringsOut->pElems == NULL) 
		{
			return E_OUTOFMEMORY;
		}
		pCaCookiesOut->pElems = reinterpret_cast<DWORD*>(CoTaskMemAlloc(sizeof(DWORD) * array.size()));
		if (pCaCookiesOut->pElems == NULL)
		{
			CoTaskMemFree(pCaStringsOut->pElems);
			return E_OUTOFMEMORY;
		}
		for (int i = 0; i < cb; i++)
		{
			pCaCookiesOut->pElems[i] = i;
			pCaStringsOut->pElems[i] = reinterpret_cast<LPOLESTR>(CoTaskMemAlloc(sizeof(OLECHAR) * (array[i].m_sProductName.length() + 1)));
			if (!pCaStringsOut->pElems[i])
			{
				while (i > 0) CoTaskMemFree(pCaStringsOut->pElems[--i]);
				CoTaskMemFree(pCaStringsOut->pElems);
				CoTaskMemFree(pCaCookiesOut->pElems);
				return E_OUTOFMEMORY;
			}
			lstrcpyW(pCaStringsOut->pElems[i], array[i].m_sProductName.c_str());
		}
		return S_OK;
	}
	case dispidRecvDataEnabled:
	{
		pCaCookiesOut->cElems = pCaStringsOut->cElems = 2;
		pCaStringsOut->pElems = reinterpret_cast<LPOLESTR*>(CoTaskMemAlloc(sizeof(LPOLESTR) * 2));
		if (pCaStringsOut->pElems == NULL) 
		{
			return E_OUTOFMEMORY;
		}
		pCaCookiesOut->pElems = reinterpret_cast<DWORD*>(CoTaskMemAlloc(sizeof(DWORD) * 2));
		if (pCaCookiesOut->pElems == NULL)
		{
			CoTaskMemFree(pCaStringsOut->pElems);
			return E_OUTOFMEMORY;
		}
		for (int i = 0; i < 2; i++)
		{
			pCaCookiesOut->pElems[i] = i;
			pCaStringsOut->pElems[i] = reinterpret_cast<LPOLESTR>(CoTaskMemAlloc(sizeof(OLECHAR) * (lstrlenW(aszBool[i]) + 1)));
			if (!pCaStringsOut->pElems[i])
			{
				while (i > 0) CoTaskMemFree(pCaStringsOut->pElems[--i]);
				CoTaskMemFree(pCaStringsOut->pElems);
				CoTaskMemFree(pCaCookiesOut->pElems);
				return E_OUTOFMEMORY;
			}
			lstrcpyW(pCaStringsOut->pElems[i], aszBool[i]);
		}
		return S_OK;
	}
	default:
		return E_NOTIMPL;
	}
}

HRESULT STDMETHODCALLTYPE CMacMonitor::GetPredefinedValue( DISPID dispID, DWORD dwCookie, VARIANT* pVarOut )
{
	if (!pVarOut) return E_POINTER;
	switch (dispID)
	{
	case dispidDeviceName:
	{
		AdapterArray& array = m_pAdapters->GetAdapterInfo();
		if (dwCookie >= array.size())
			return E_INVALIDARG;
		VariantInit(pVarOut);
		pVarOut->vt = VT_BSTR;
		pVarOut->bstrVal = SysAllocString(array[dwCookie].m_sProductName.c_str());
		return S_OK;
	}
	case dispidRecvDataEnabled:
	{
		if (dwCookie > 1) return E_INVALIDARG;
		pVarOut->vt = VT_BOOL;
		pVarOut->boolVal = (dwCookie) ? VARIANT_FALSE : VARIANT_TRUE;
	}
	default:
		return E_NOTIMPL;
	}
}
#endif

HRESULT CMacMonitor::IPersistStreamInit_Load(LPSTREAM pStm, ATL_PROPMAP_ENTRY* pMap)
{
	m_fLoading = true;
	ATLTRACE(_T("Start PersistStreamInit_Load\n"));
	HRESULT hr = AtlIPersistStreamInit_Load(pStm, pMap, this, GetUnknown());
	if (SUCCEEDED(hr))
		m_bRequiresSave = FALSE;
	m_fLoading = false;
	return hr;
}
///////////////////////////////////////////////////////////////////////////////////////////////

HRESULT STDMETHODCALLTYPE CMacMonitor::Start( void)
{
	if (m_bValid == FALSE) return Error(IDS_E_DEVICE);
	if (m_fMonitoring) return S_OK;

	if (m_hThread == NULL)
	{
		HRESULT hr = CreateEventWindow();
		if (hr != S_OK) return hr;
		ResetEvent();
		m_hThread = CreateThread(NULL, 0, RecvThreadProc, this, 0, &m_idThread);
		if (m_hThread == NULL)
			return E_OUTOFMEMORY;
		SetThreadPriority(m_hThread, THREAD_PRIORITY_HIGHEST);
		WaitForSingleObject(m_hThreadSync, INFINITE);
	}
	ResetEvent();
	m_hResult = S_OK;
	m_lPackets = m_lBytes = 0;
	PostThreadMessage(m_idThread, m_uiStartMonitor, 0, 0);
	WaitForSingleObject(m_hThreadSync, INFINITE);
	if (m_lMonitorEventInterval)
	{
		m_lLastTick = GetTickCount();
		SetTimer(ID_TMR, m_lMonitorEventInterval);
	}
	m_fMonitoring = true;
	return m_hResult;
}
        
HRESULT STDMETHODCALLTYPE CMacMonitor::Stop( void)
{
	if (!m_fMonitoring) return S_OK;

	KillTimer(ID_TMR);
	ResetEvent();
	PostThreadMessage(m_idThread, m_uiStopMonitor, 0, 0);
	WaitForSingleObject(m_hThreadSync, INFINITE);
	m_fMonitoring = false;
	Clear();
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacMonitor::RemoveFilters( void)
{
	if (m_fMonitoring) return Error(IDS_E_ALREADYSTART);
	for (AFilterIter it = m_afilter.begin(); it != m_afilter.end(); it++)
		delete (*it);
	m_afilter.clear();
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacMonitor::AddFilter( 
            /* [in] */ IFilterParam __RPC_FAR *pParam)
{
	if (!pParam) return E_POINTER;
	if (m_fMonitoring) return Error(IDS_E_ALREADYSTART);
	IFilterParam* pFiltervtbl;
	HRESULT hr = pParam->QueryInterface(IID_IFilterParam, (void**)&pFiltervtbl);
	if (hr != S_OK) return hr;
	CFilter* pFilter = CFilter::CreateFilter(pFiltervtbl);
	pFiltervtbl->Release();
	if (pFilter == NULL)
		return Error(IDS_E_BADPARAM);
	m_afilter.push_back(pFilter);
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CMacMonitor::UnpackHelper( 
            /* [in] */ VARIANT v,
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	HRESULT hr = Unpack(v, pResult);
	if (hr != S_OK && HRESULT_FACILITY(hr) == FACILITY_NULL)
		return Error(HRESULT_CODE(hr));
	return hr;
}

HRESULT STDMETHODCALLTYPE CMacMonitor::put_RecvDataEnabled( 
            /* [in] */ VARIANT_BOOL NewVal)
{
	bool f = (NewVal) ? true : false;
	if (f != m_fRecvDataEnabled)
	{
		FireOnRequestEdit(dispidRecvDataEnabled);
		m_fRecvDataEnabled = f;
		SetDirty(TRUE);
		FireOnChanged(dispidRecvDataEnabled);
	}
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CMacMonitor::get_RecvDataEnabled( 
            /* [retval][out] */ VARIANT_BOOL __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	*pRetVal = (m_fRecvDataEnabled) ? VARIANT_TRUE : VARIANT_FALSE;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CMacMonitor::put_MonitorEventInterval( 
            /* [in] */ long NewVal)
{
	if (m_lMonitorEventInterval != NewVal)
	{
		FireOnRequestEdit(dispidMonitorEventInterval);
		m_lMonitorEventInterval = NewVal;
		SetDirty(TRUE);
		FireOnChanged(dispidMonitorEventInterval);
		if (m_fMonitoring)
			SetTimer(ID_TMR, NewVal);
	}
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacMonitor::get_MonitorEventInterval( 
            /* [retval][out] */ long __RPC_FAR *pRetVal)
{
	if (!pRetVal) return E_POINTER;
	*pRetVal = m_lMonitorEventInterval;
	return S_OK;
}

HANDLE CMacMonitor::OpenAdapter()
{
	AdapterArray& array = m_pAdapters->GetAdapterInfo();
	for (AdapterArrayIter it = array.begin(); it != array.end(); it++)
	{
		if (m_szDevice == (*it).m_sProductName
			|| m_szDevice == (*it).m_sServiceName)
		{
			HANDLE h = CreateFileW((*it).m_sDeviceName.c_str(), 
				GENERIC_READ, 0, NULL, 
				CREATE_ALWAYS, FILE_FLAG_OVERLAPPED, NULL);
			if (h == INVALID_HANDLE_VALUE)
			{
				ATLTRACE(_T("Open Failed %d\n"), GetLastError());
			}
			return h;
		}
	}
	return INVALID_HANDLE_VALUE;
}

HRESULT STDMETHODCALLTYPE CMacMonitor::put_DeviceName( 
            /* [in] */ BSTR NewValue)
{
	if (m_fLoading) return S_OK;	// Ignore
	if (!m_pAdapters) return E_NOINTERFACE;
	if (!NewValue) return Error(IDS_E_NOSUCHDEVICE);

	AdapterArray& array = m_pAdapters->GetAdapterInfo();
	bool fFound(false);
	for (AdapterArrayIter it = array.begin(); it != array.end(); it++)
	{
		if (lstrcmpiW(NewValue, (*it).m_sProductName.c_str()) == 0
			|| lstrcmpiW(NewValue, (*it).m_sServiceName.c_str()) == 0)
		{
			fFound = true;
			break;
		}
	}
	if (!fFound) 
	{
		BOOL b;
		if (GetAmbientUserMode(b) == S_OK && b)
			return Error(IDS_E_NOSUCHDEVICE);
		else
			m_bValid = FALSE;
	}

	if (m_szDevice != NewValue)
	{
		FireOnRequestEdit(dispidDeviceName);
		m_szDevice = NewValue;
		SetDirty(TRUE);
		FireOnChanged(dispidDeviceName);
	}

	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacMonitor::get_DeviceName( 
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	*pResult = SysAllocString(m_szDevice.c_str());
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CMacMonitor::Clear( void)
{
	while (m_listPackets.empty() == false)
	{
		IMacPacket* p = m_listPackets.front();
		m_listPackets.pop_front();
		p->Release();
	}
	return S_OK;
}
