// Copyright(c) 1999 arton

#include "stdafx.h"
#include "MacCapv2.h"
#include "ProtocolNameCollection.h"
#include "NameValuePair.h"
#include "AssNum.h"

class CProtoEnum : public IEnumVARIANT
{
public:
	CProtoEnum(int iIndex = 0)
		: m_ulRef(1), m_iIndex(iIndex)
	{
	}
	~CProtoEnum()
	{
	}
	STDMETHOD_(ULONG, AddRef)()
	{
		return ++m_ulRef;
	}
	STDMETHOD_(ULONG, Release)()
	{
		m_ulRef--;
		if (m_ulRef <= 0)
		{
			delete this;
			return 0;
		}
		return m_ulRef;
	}
	STDMETHOD(QueryInterface)(REFIID iid, void ** ppvObject)
	{
		if (!ppvObject) return E_POINTER;
		if (IsEqualIID(IID_IUnknown, iid) 
			|| IsEqualIID(IID_IEnumVARIANT, iid)) {
			*ppvObject = this;
			m_ulRef++;
			return S_OK;
		}
		return E_NOINTERFACE;
	}

    HRESULT STDMETHODCALLTYPE Next(
		ULONG cElements, 
		VARIANT FAR* pvar, 
		ULONG FAR* pcElementFetched)
	{
		if (m_iIndex >= MAX_PROTO) return S_FALSE;

		ULONG l2;
		if (pcElementFetched != NULL)        
			*pcElementFetched = 0;
		for (ULONG l=0; l<cElements; l++)        
			VariantInit(&pvar[l]);
		// Retrieve the next cElements elements.
		for (l2 = 0; m_iIndex < MAX_PROTO && l2 < cElements; l2++, m_iIndex++)
		{
			for (; m_iIndex < MAX_PROTO; m_iIndex++)
			{
				if (IsAssigned((BYTE)m_iIndex)) break;
			}
			if (m_iIndex >= MAX_PROTO) break;

			CComObject<CNameValuePair>* p;
			HRESULT hr = CComObject<CNameValuePair>::CreateInstance(&p);
			if (hr != S_OK)
				break;
			p->AddRef();
			if (p->QueryInterface(IID_INameValuePair, (void**)&pvar[l2].pdispVal) != S_OK)
			{
				p->Release();
				break;
			}
			p->m_strName = GetProtoName((BYTE)m_iIndex);
			p->m_value.vt = VT_I2;
			p->m_value.iVal = (short)m_iIndex;
			pvar[l2].vt = VT_DISPATCH;
		}    
		// Set count of elements retrieved.
		if (pcElementFetched != NULL)        
			*pcElementFetched = l2;
		return  (l2 < cElements) ? S_FALSE : S_OK;
	}
    HRESULT STDMETHODCALLTYPE Skip(unsigned long celt)
	{
		while (celt)
		{
			if (m_iIndex > MAX_PROTO) return S_FALSE;
			celt--;
			m_iIndex++;
		}
		return S_OK;
	}
    HRESULT STDMETHODCALLTYPE Reset()
	{
		m_iIndex = 0;
		return S_OK;
	}
    HRESULT STDMETHODCALLTYPE Clone(IEnumVARIANT FAR* FAR* ppenum)
	{
		if (!ppenum) return E_POINTER;
		*ppenum = new CProtoEnum(m_iIndex);
		return S_OK;
	}
private:
	ULONG m_ulRef;
	int m_iIndex;
};

/////////////////////////////////////////////////////////////////////////////
// CProtocolNameCollection

HRESULT STDMETHODCALLTYPE CProtocolNameCollection::get__NewEnum( 
            /* [retval][out] */ IUnknown __RPC_FAR *__RPC_FAR *ppUnk)
{
	if (!ppUnk) return E_POINTER;
	*ppUnk = new CProtoEnum;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CProtocolNameCollection::get_Length( 
            /* [retval][out] */ long __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	*pResult = GetIPNameTableSize();
	return S_OK;
}
        
HRESULT STDMETHODCALLTYPE CProtocolNameCollection::Find( 
            /* [in] */ long Proto,
            /* [retval][out] */ BSTR __RPC_FAR *pResult)
{
	if (!pResult) return E_POINTER;
	*pResult = SysAllocString(GetProtoName((BYTE)Proto));
	return S_OK;
}

