// Copyright(c) 1999 arton
//	FreePeek packet description pretty printer
function show_packet(packet)
{
	print_time(packet);
	print_macheader(packet);

	switch (packet.EtherType)
	{
	case 2048:	// IP
		print_ip(packet);
		break;
	case 2054:	// ARP
	case 32821:	// RARP
		print_arp(packet);
		break;
	default:
		dumpall(packet.UnpackedString.substr(28));
		break;
	}
}

function rgb(red, green, blue)
{
	return (blue << 16) + (green << 8) + red;
}

function padstr(s, l)
{
	var str = s.toString();
	var ln = str.length;
	if (ln > l)
		return str.substr(0, l);
	else if (ln == l)
		return str;
	for (i = 0; i < l - ln; i++)
		str = "0" + str;
	return str;
}

function print_time(packet)
{
	var d = new Date(packet.RecvTime);
	var s = padstr(d.getHours(), 2) + ":";
	s += padstr(d.getMinutes(), 2) + ":";
	s += padstr(d.getSeconds(), 2) + ":";
	s += padstr(d.getMilliseconds(), 3);

	document.writeln("Time = " + s + ", Length = " + packet.Length);
}

function print_macheader(packet)
{
	var s;

	document.color = rgb(32, 128, 64);
	document.writeln("");
	document.writeln("MAC: Destination Address: " + packet.DestMacString);
	if (packet.IsMulticast)
		s = "MAC:                      Multicast Address";
	else
		s = "MAC:                           ";
	s += "(" + packet.DestVendorString + ")";
	document.writeln(s);
	document.writeln("MAC:      Source Address: " + packet.SrcMacString);
	s = "MAC:                           ";
	s += "(" + packet.SrcVendorString + ")";
	document.writeln(s);
	document.writeln("MAC:       Ethernet Type: " + packet.EtherType);
	try
	{
		document.writeln("MAC:                     (" + packet.EtherTypeString + ")");
	}
	catch (e)
	{
		document.writeln("MAC:                     (Unknown)");
	}
}

function print_ip(packet)
{
	document.color = rgb(0, 64, 64);
	document.writeln("");

	var dat = packet.UnpackedString.substr(28);
	var hlen = dat.charAt(1);
	var tlen = parseInt(dat.substr(4, 4), 16);
	var s = "IP: Version: " + dat.charAt(0) + ", Header Length: " + hlen + ", Total Length: " + tlen;
	document.writeln(s);
	document.writeln("IP: Service Type:");
	var tos = parseInt(dat.substr(2, 2), 16);
	document.writeln("IP:         Precedence: " + (tos >> 5));
	document.writeln("IP:");
	document.writeln("IP:              Delay: " + ((tos >> 4) & 1));
	document.writeln("IP:         Throughput: " + ((tos >> 3) & 1));
	document.writeln("IP:        Reliablitiy: " + ((tos >> 2) & 1));
	document.writeln("IP:           Low-cost: " + ((tos >> 1) & 1));
	document.writeln("IP:");
	document.writeln("IP: Flagmentation Control:");
	document.writeln("IP:       Identification: 0x" + dat.substr(8, 4) + "(" + parseInt(dat.substr(8, 4), 16) + ")");
	var b = parseInt(dat.substr(12, 1), 16);
	document.writeln("IP:        ( reserved )   bit: " + ((b >> 3) & 1));
	document.writeln("IP:       do not fragment bit: " + ((b >> 2) & 1));
	document.writeln("IP:        more fragments bit: " + ((b >> 1) & 1));
	document.writeln("IP:       Fragment Offset: " + parseInt(dat.substr(13, 3), 16) * 8);
	document.writeln("IP:");
	document.writeln("IP: Time To Live: " + parseInt(dat.substr(16, 2), 16) + " (Seconds/Hops)");
	s = "IP: Protocol: 0x" + dat.substr(18, 2) + "(";
	try
	{
		s = s + packet.InternetProtocolString;
	}
	catch (e)
	{
	}
	document.writeln(s + ")");
	document.writeln("IP: Header Checksum: " + dat.substr(20, 4));
	document.writeln("IP:      Source Address = " + packet.SrcIPAddressString);
	document.writeln("IP: Destination Address = " + packet.DestIPAddressString);
	if ((hlen - 5) != 0)
	{
		document.writeln("IP: Options Data");
		document.dump(dat.substr(40, (hlen - 5) * 4 * 2), 20);
	}

	switch (parseInt(dat.substr(18, 2), 16))
	{
	case 1:	// ICMP
		print_icmp(dat.substr(hlen * 8, tlen * 2 - hlen * 8));
		break;
	case 6:	// TCP
		print_tcp(dat.substr(hlen * 8, tlen * 2 - hlen * 8));
		break;
	case 17:	// UDP
		print_udp(dat.substr(hlen * 8, tlen * 2 - hlen * 8));
		break;
	default:
		dumpall(dat.substr(hlen * 8, tlen * 2 - hlen * 8));
		break;
	}
}

function print_icmp(dat)
{
	var len = dat.length / 2;
	var s;
	document.color = rgb(128, 32, 200);
	document.writeln("");

	var cd = parseInt(dat.substr(0, 2), 16);
	document.writeln("ICMP: " + dat.substr(0, 2) + " (" + document.icmptype(cd) + ")");
	switch (cd)
	{
	case 8:
	case 0:
		// Echo
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("ICMP:         ID: " + dat.substr(8, 4) + ", sequence: " + parseInt(dat.substr(12, 4), 16));
		document.writeln("");
		document.writeln("optional data: (" + (len - 8) + "bytes)");
		if ((len - 8) > 0)
			document.dump(dat.substr(16, (len - 8) * 2), 0);
		break;
	case 3:
		// Unreachable
		cd = parseInt(dat.substr(2, 2), 16);
		s = "ICMP:   Code: " + cd + " (";
		switch (cd)
		{
		case 0:
			s = s + "Network Unreachable";
			break;
		case 1:
			s = s + "Host Unreachable";
			break;
		case 2:
			s = s + "Protocol Unreachable";
			break;
		case 3:
			s = s + "Port Unreachable";
			break;
		case 4:
			s = s + "Fragmentation Needed and DF set";
			break;
		case 5:
			s = s + "Source Route Failed";
			break;
		case 6:
			s = s + "Destination network unknown";
			break;
		case 7:
			s = s + "Destination host unknown";
			break;
		case 8:
			s = s + "Source host isolated";
			break;
		case 9:
			s = s + "Communication with destination network administratively prohibited";
			break;
		case 10:
			s = s + "Communication with destination host Administratively prohibited";
			break;
		case 11:
			s = s + "Network unreachable for type of service";
			break;
		case 12:
			s = s + "Host unreachable for type of service";
			break;
		default:
			s = s + "Unknown Code";
			break;
		}
		document.writeln(s + ")");
		// fall down to the next case
	case 4:
		// Source Quench
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("");
		document.writeln("Datagram:");
		if (len - 8 > 0)
			document.dump(dat.substr(16, (len - 8) * 2), 0);
		break;
	case 5:
		// route change
		cd = parseInt(dat.substr(2, 2), 16);
		s = "ICMP:   Code: " + cd + " (";
		switch (cd)
		{
		case 0:
			s = s + "Redirect datagrams for the Network (obsoleted)";
			break;
		case 1:
			s = s + "Redirect datagrams for the Host";
			break;
		case 2:
			s = s + "Redirect datagrams for the Type of Service and Network";
			break;
		case 3:
			s = s + "Redirect datagrams for the Type of Service and Host";
			break;
		default:
			s = s + "Unknown Code";
			break;
		}
		document.writeln(s + ")");
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("ICMP:   Gateway IP Address: " + parseInt(dat.substr(8, 2), 16) + "."
					     + parseInt(dat.substr(10, 2), 16) + "."
					     + parseInt(dat.substr(12, 2), 16) + "."
					     + parseInt(dat.substr(14, 2), 16));
		document.writeln("");
		document.writeln("Datagram:");
		if (len - 8 > 0)
			document.dump(dat.substr(16, (len - 8) * 2), 0);
		break;
	case 11:
		// Detecting Circular
		cd = parseInt(dat.substr(2, 2), 16);
		switch (cd)
		{
		case 0:
			document.writeln("ICMP:   Code: 0 (Time-to-live count exceeded)");
			break;
		case 1:
			document.writeln("ICMP:   Code: 1 (Fragment reassembly time exceeded)");
			break;
		default:
			document.writeln("ICMP:   Code: " + cd + " (Unknown)");
			break;
		}
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("");
		document.writeln("Datagram:");
		if (len - 8 > 0)
			document.dump(dat.substr(16, (len - 8) * 2), 0);
		break;
	case 12:
		// Other problem
		cd = parseInt(dat.substr(2, 2), 16);
		switch (cd)
		{
		case 0:
			document.writeln("ICMP:   Code: 0, Pointer="
				+ parseInt(dat.substr(8, 2), 16) + "(0x" + dat.substr(8, 2) + ") indicates the error");
			break;
		case 1:
			document.writeln("ICMP:   Code: 1 (Missing a Required Option)");
			break;
		case 2:
			document.writeln("ICMP:   Code: 2 (Bad Length)");
			break;
		default:
			document.writeln("ICMP:   Code: " + cd + " (Unknown, pointer="
				+ parseInt(dat.substr(8, 2), 16) + "(0x" + dat.substr(8, 2) + ")");
			break;
		}
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("");
		document.writeln("Datagram:");
		if (len - 8 > 0)
			document.dump(dat.substr(16, (len - 8) * 2), 0);
		break;
	case 13:
	case 14:
		// Clock Synch
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("ICMP:                  ID: " + dat.substr(8, 4) + ", sequence: " + parseInt(dat.substr(12, 4), 16));
		document.writeln("ICMP: originate timestamp: " + parseInt(dat.substr(16, 8), 16) + "(" + dat.substr(16, 8) + ")");
		if (parseInt(dat.substr(0, 2), 16) == 14)
		{
			document.writeln("ICMP:   receive timestamp: " + parseInt(dat.substr(24, 8), 16) + "(" + dat.substr(24, 8) + ")");
			document.writeln("ICMP:  transmit timestamp: " + parseInt(dat.substr(32, 8), 16) + "(" + dat.substr(32, 8) + ")");
		}
		break;
	case 17:
		// Obtaining Subnet request
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		break;
	case 18:
		// Obtaining Subnet reply
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		document.writeln("ICMP:  ADDRESS MASK: " + parseInt(dat.substr(8, 2), 16) + "."
					     + parseInt(dat.substr(10, 2), 16) + "."
					     + parseInt(dat.substr(12, 2), 16) + "."
					     + parseInt(dat.substr(14, 2), 16));
		break;
	default:
		s = "ICMP:   Code: " + parseInt(dat.substr(2, 2), 16);
		document.writeln("ICMP:   Checksum: " + parseInt(dat.substr(4, 4), 16));
		break;
	}
}

function print_udp(dat)
{
	document.color = rgb(128, 32, 200);
	document.writeln("");

	var is = parseInt(dat.substr(0, 4), 16);
	var ss = document.udpport(is);
	if (ss == "")
		ss = "-";
	var id = parseInt(dat.substr(4, 4), 16);
	var sd = document.udpport(id);
	if (sd == "")
		sd = "-";
	document.writeln("UDP:      Source Port: " + is + "(" + ss + ")");
	document.writeln("UDP: Destination Port: " + id + "(" + sd + ")");
	document.writeln("UDP:         Checksum: 0x" + dat.substr(12, 4));

	if (is == 53 || id == 53)
		print_domain(dat.substr(16));
	else
		dumpall(dat.substr(16));
}

function print_tcp(dat)
{
	document.color = rgb(128, 32, 200);
	document.writeln("");
	
	var is = parseInt(dat.substr(0, 4), 16);
	var ss = document.tcpport(is);
	if (ss == "")
		ss = "-";
	var id = parseInt(dat.substr(4, 4), 16);
	var sd = document.tcpport(id);
	if (sd == "")
		sd = "-";

	document.writeln("TCP:      Source Port: " + is + "(" + ss + ")");
	document.writeln("TCP: Destination Port: " + id + "(" + sd + ")");
	document.writeln("TCP:         Sequence Number: " + parseInt(dat.substr(8, 8), 16));
	document.writeln("TCP:  Acknowledgement Number: " + parseInt(dat.substr(16, 8), 16));
	var hlen = parseInt(dat.substr(24, 1), 16) * 4;
	document.writeln("TCP:           Header Length: " + hlen);
	document.writeln("TCP:");
	document.writeln("TCP: Code Bits");
	var b = parseInt(dat.substr(26, 2), 16);
	document.writeln("TCP:                     URG: " + ((b >> 5) & 1));
	document.writeln("TCP:                     ACK: " + ((b >> 4) & 1));
	document.writeln("TCP:                     PSH: " + ((b >> 3) & 1));
	document.writeln("TCP:                     RST: " + ((b >> 2) & 1));
	document.writeln("TCP:                     SYN: " + ((b >> 1) & 1));
	document.writeln("TCP:                     FIN: " + (b & 1));
	document.writeln("TCP:");
	document.writeln("TCP:                  Window: " + parseInt(dat.substr(28, 4), 16));
	document.writeln("TCP:                Checksum: " + parseInt(dat.substr(32, 4), 16));
	document.writeln("TCP:          Urgent Pointer: " + parseInt(dat.substr(36, 4), 16));

	if (hlen > 20)
	{
		document.writeln("TCP:");
		document.writeln("TCP: Options: (" + (hlen - 20) + "bytes)");
		document.dump(dat.substr(40, (hlen - 20) * 2), 0);
	}

	dumpall(dat.substr(hlen * 2));
}

function print_arp(packet)
{
	document.color = rgb(0, 64, 64);
	document.writeln("");

	var dat = packet.UnpackedString.substr(28);
	document.writeln("ARP: Hardware Type: " + dat.substr(0, 4) + ", Protocol Type: " + dat.substr(4, 4));
	document.writeln("ARP: Hardware Length = " + parseInt(dat.substr(8, 2), 16) + ", Protocol Length = " + parseInt(dat.substr(10, 2), 16));
	var s = "ARP: Operation = ";
	switch (parseInt(dat.substr(12, 4), 16))
	{
	case 1:
		s = s + "ARP request";
		break;
	case 2:
		s = s + "ARP response";
		break;
	case 3:
		s = s + "RARP request";
		break;
	case 4:
		s = s + "RARP response";
		break;
	case 5:
		s = s + "Dynamic RARP request";
		break;
	case 6:
		s = s + "Dynamic RARP reply";
		break;
	case 7:
		s = s + "Dynamic RARP error";
		break;
	case 8:
		s = s + "InARP request";
		break;
	case 9:
		s = s + "InARP resply";
		break;
	default:
		s = s + "Unknown Operation";
		break;
	}
	document.writeln(s);
	document.writeln("ARP: Sender HA = " + dat.substr(16, 2) + "-" 
					     + dat.substr(18, 2) + "-" 
					     + dat.substr(20, 2) + "-" 
					     + dat.substr(22, 2) + "-" 
					     + dat.substr(24, 2) + "-" 
					     + dat.substr(26, 2));
	document.writeln("ARP: Sender IP = " + parseInt(dat.substr(28, 2), 16) + "."
					     + parseInt(dat.substr(30, 2), 16) + "."
					     + parseInt(dat.substr(32, 2), 16) + "."
					     + parseInt(dat.substr(34, 2), 16));
	document.writeln("ARP: Target HA = " + dat.substr(36, 2) + "-" 
					     + dat.substr(38, 2) + "-" 
					     + dat.substr(40, 2) + "-" 
					     + dat.substr(42, 2) + "-" 
					     + dat.substr(44, 2) + "-" 
					     + dat.substr(46, 2));
	document.writeln("ARP: Sender IP = " + parseInt(dat.substr(48, 2), 16) + "."
					     + parseInt(dat.substr(50, 2), 16) + "."
					     + parseInt(dat.substr(52, 2), 16) + "."
					     + parseInt(dat.substr(54, 2), 16));
}

function dumpall(data)
{
	document.color = rgb(0, 64, 200);
	document.writeln("");
	document.writeln("Data: " + (data.length / 2) + "Bytes");
	document.dump(data, 0);
}

// RFC 1035
function print_domain(dat)
{
	var s;
	document.color = rgb(32, 32, 128);
	document.writeln("");
	document.writeln("DNS:     ID: 0x" + dat.substr(0, 4)); 
	var flag = parseInt(dat.substr(4, 4), 16);
	if (flag >>> 15)
		document.writeln("DNS:      R: 1 - response");
	else
		document.writeln("DNS:      Q: 0 - query");
	switch ((flag & 0x7800) >> 11)
	{
	case 0:
		s = "0 - Standard query (name to address)";
		break;
	case 1:
		s = "1 - Inverse query (address to name)";
		break;
	case 2:
		s = "2 - Server status request";
		break;
	default:
		s = ((flag & 0x7800) >> 11) + " - Unknwon";
		break;
	}
	document.writeln("DNS: OPCode: " + s);
	document.writeln("DNS:     AA: " + ((flag & 0x400) >> 10));
	document.writeln("DNS:     TC: " + ((flag & 0x200) >> 9));
	document.writeln("DNS:     RD: " + ((flag & 0x100) >> 8));
	document.writeln("DNS:     RA: " + ((flag & 0x80) >> 7));
	switch (flag & 0x0f)
	{
	case 0:
		s = "0 - No error condition";
		break;
	case 1:
		s = "1 - Unable to interpret query due to format error";
		break;
	case 2:
		s = "2 - Unable to process due to server failure";
		break;
	case 3:
		s = "3 - Name in query does not exist";
		break;
	case 4:
		s = "4 - Type of query not supported";
		break;
	case 5:
		s = "5 - Query refused";
		break;
	default:
		s = (flag & 0x0f) + " - Unknwon";
		break;
	}
	document.writeln("DNS: RCode: " + s);
	document.writeln("DNS:");
	var qcount = parseInt(dat.substr(8, 4), 16);
	var acount = parseInt(dat.substr(12, 4), 16);
	var aucount = parseInt(dat.substr(16, 4), 16);
	var adcount = parseInt(dat.substr(20, 4), 16);
	document.writeln("DNS:   Question conunt = " + qcount);
	document.writeln("DNS:     Answer conunt = " + acount);
	document.writeln("DNS:  Authority conunt = " + aucount);
	document.writeln("DNS: Additional conunt = " + adcount);

	var offset = 24;
	while (qcount--)
	{
		document.writeln("DNS:");
		var i = print_domainname(dat, offset);
		document.writeln("DNS:  QNAME: " + print_domainname.domainname);
		document.writeln("DNS:  QTYPE: " + get_dns_rrtype(dat.substr(i, 4)));
		i += 4;
		document.writeln("DNS: QCLASS: " + get_dns_rrclass(dat.substr(i, 4)));
		offset = i + 4;
	}

	if (acount > 0)
	{
		document.color = rgb(0, 0, 0);
		document.write("\r\nDNS: Answer count=" + acount);
		while (acount-- > 0)
		{
			offset = print_dnsans(dat, offset);
		}
	}
	if (aucount > 0)
	{
		document.color = rgb(32, 64, 32);
		document.writeln("\r\nDNS: Authority count=" + aucount);
		while (aucount-- > 0)
		{
			offset = print_dnsans(dat, offset);
		}
	}
	if (adcount > 0)
	{
		document.color = rgb(128, 32, 32);
		document.writeln("\r\nDNS: Additional count=" + adcount);
		while (adcount-- > 0)
		{
			offset = print_dnsans(dat, offset);
		}
	}
}

function print_dnsans(dat, offset)
{
	document.writeln("DNS:");
	var i = print_domainname(dat, offset);
	document.writeln("DNS:   NAME: " + print_domainname.domainname);
	var rrtype = dat.substr(i, 4);
	document.writeln("DNS:   TYPE: " + get_dns_rrtype(rrtype));
	i += 4;
	var rrclass = dat.substr(i, 4);
	document.writeln("DNS:  CLASS: " + get_dns_rrclass(rrclass));
	i += 4;
	document.writeln("DNS:    TTL: " + parseInt(dat.substr(i, 8), 16));
	i += 8;
	var rlen = parseInt(dat.substr(i, 4), 16);
	s = "DNS:  RDLen: " + rlen;
	i += 4;
	rlen *= 2;
	if (rlen > 0)
	{
		if (rrtype == "0001" && rrclass == "0001")
		{
			document.writeln("DNS: Address: " 
				+ parseInt(dat.substr(i, 2), 16) + "." + parseInt(dat.substr(i + 2, 2), 16) + "."
				+ parseInt(dat.substr(i + 4, 2), 16) + "." + parseInt(dat.substr(i + 6, 2), 16));
		}
		else if ((rrtype == "0002" || rrtype == "000C") && rrclass == "0001")
		{
			print_domainname(dat, i);
			document.writeln("DNS: domain name: " + print_domainname.domainname);
		}
		else if (rrtype == "000F" && rrclass == "0001")
		{
			document.writeln("DNS: Prefference: " + parseInt(dat.substr(i, 4), 16));
			print_domainname(dat, i + 4);
			document.writeln("DNS: domain name: " + print_domainname.domainname);
		}
		else
		{
			document.writeln(s);
			document.dump(dat.substr(i, rlen), 0);
		}
	}
	i += rlen;
	return i;
}

function print_domainname(dat, offset)
{
	var i = offset;
	var s = "";
	for (;;)
	{
		var n = parseInt(dat.substr(i, 2), 16);
		i += 2;
		if (n >= 0xc0)
		{
			n = parseInt(dat.substr(i - 2, 4), 16) - 0xc000;
			if (i > offset + 2)
				s += ".";
			i += 2;
			print_domainname(dat, n * 2);
			s += print_domainname.domainname;
			break;
		}
		else if (n > 0)
		{
			n *= 2;
			if (i > offset + 2)
				s += ".";
			for (j = i; j < n + i; j += 2)
				s += String.fromCharCode(parseInt(dat.substr(j, 2), 16));
			i += n;
		}
		else
		{
			break;
		}
	}
	print_domainname.domainname = s;
	return i;
}

function get_dns_rrtype(dat)
{
	var s = "";
	switch (parseInt(dat, 16))
	{
	case 1:
		s = "A  - a host address";
		break;
	case 2:
		s = "NS - an authoritative name server"
		break;
	case 3:
		s = "MD - a mail destination (Obsolete - use MX)";
		break;
	case 4:
		s = "MF - a mail forwarder (Obsolete - use MX)";
		break;
	case 5:
		s = "CNAME - the canonical name for an alias";
		break;
	case 6:
		s = "SOA - marks the start of a zone of authority";
		break;
	case 7:
		s = "MB - a mailbox domain name (EXPERIMENTAL)";
		break;
	case 8:
		s = "MG - a mail group member (EXPERIMENTAL)";
		break;
	case 9:
		s = "MR - a mail rename domain name (EXPERIMENTAL)";
		break;
	case 10:
		s = "NULL - a null RR (EXPERIMENTAL)";
		break;
	case 11:
		s = "WKS - a well known service description";
		break;
	case 12:
		s = "PTR - a domain name pointer";
		break;
	case 13:
		s = "HINFO - host information";
		break;
	case 14:
		s = "MINFO - mailbox or mail list information";
		break;
	case 15:
		s = "MX - mail exchange";
		break;
	case 16:
		s = "TXT - text strings";
		break;
	case 252:
		s = "AXFR - A request for a transfer of an entire zone";
		break;
	case 253:
		s = "MAILB - A request for mailbox-related records (MB, MG or MR)";
		break;
	case 254:
		s = "MAILA - A request for mail agent RRs (Obsolete - see MX)";
		break;
	case 255:
		s = "* - A request for all records";
		break;
	default:
		s = "Unknown type (0x" + dat + ")";
		break;
	}
	return s;
}

function get_dns_rrclass(dat)
{
	var s = "";
	switch (parseInt(dat, 16))
	{
	case 1:
		s = "IN - the Internet";
		break;
	case 2:
		s = "CS - the CSNET class (Obsolete - used only for examples in some obsolete RFCs)";
		break;
	case 3:
		s = "CH - the CHAOS class";
		break;
	case 4:
		s = "HS - Hesiod [Dyer 87]";
		break;
	case 255:
		s = "*  - any class";
		break;
	default:
		s = "Unknown class (0x" + dat + ")";
		break;
	}
	return s;
}

